// cgi.cpp
// copyright (C) 2000 by David Cox All Rights Reserved

#include "cgi.h"
#include "cereserror.h"
#include <stdio.h>
#include <iostream.h>
#include <stdlib.h>
#include <memory.h>
#include <string.h>
#include <direct.h>


Env::Env()
{
	m_gatewayInterface[0] =		0;
	m_serverName[0] =			0;
	m_serverPort[0] =			0;
	m_serverProtocol[0] =		0;
	m_serverSoftware[0] =		0;

	m_authType[0] =				0;
	m_contentFile[0] =			0;
	m_contentLength[0] =		0;
	m_contentType[0] =			0;
	m_outputFile[0] =			0;
	m_pathInfo[0] =				0;
	m_pathTranslated[0] =		0;
	m_queryString[0] =			0;
	m_remoteAddr[0] =			0;
	m_remoteUser[0] =			0;
	m_requestLine[0] =			0;
	m_requestMethod[0] =		0;
	m_scriptName[0] =			0;

	m_accept[0] =				0,
	m_referrer[0] =				0;
	m_userAgent[0] =			0;
	m_cookie[0] =				0;
	m_query =					0;

	memset(m_dir, 0, e_dirSize);
	_getcwd(m_dir, e_dirSize);
}


Env::~Env()
{
	if (m_query) delete [] m_query;
}


void Env::Print()
{
	cout << "<P>";
	cout << "Gateway Interface: " << m_gatewayInterface << "<BR>" << endl;
	cout << "Server Name:       " << m_serverName << "<BR>" << endl;
	cout << "Server Port:       " << m_serverPort << "<BR>" << endl;
	cout << "Server Protocol:   " << m_serverProtocol << "<BR>" << endl;
	cout << "Server Software:   " << m_serverSoftware << "<BR>" << endl;

	cout << "Auth Type:         " << m_authType << "<BR>" << endl;
	cout << "Content File:      " << m_contentFile << "<BR>" << endl;
	cout << "Content Length:    " << m_contentLength << "<BR>" << endl;
	cout << "Content Type:      " << m_contentType << "<BR>" << endl;
	cout << "Output File:       " << m_outputFile << "<BR>" << endl;
	cout << "Path Info:         " << m_pathInfo << "<BR>" << endl;
	cout << "Path Translated:   " << m_pathTranslated << "<BR>" << endl;
	cout << "Query String:      " << m_queryString << "<BR>" << endl;
	cout << "Remote Addr:       " << m_remoteAddr << "<BR>" << endl;
	cout << "Remote User:       " << m_remoteUser << "<BR>" << endl;
	cout << "Request Line:      " << m_requestLine << "<BR>" << endl;
	cout << "Request Method:    " << m_requestMethod << "<BR>" << endl;
	cout << "Script Name:       " << m_scriptName << "<BR>" << endl;

	cout << "From:              " << m_httpFrom << "<BR>" << endl;
	cout << "Accept:            " << m_accept << "<BR>" << endl;
	cout << "Referrer:          " << m_referrer << "<BR>" << endl;
	cout << "User Agent:        " << m_userAgent << "<BR>" << endl;
	cout << "Cookie:            " << m_cookie << "<BR>" << endl;
	cout << "Dir:               " << m_dir << "<BR>" << endl;
	cout << "Query:             " << m_query << "<BR>" << endl;
	cout << "</P>";
}


Cgi::Cgi()
{
}


Cgi::~Cgi()
{
}


//*******************************************************
// Initialization routines
//*******************************************************

// copy a single environment variable
void Cgi::GetEnv(char * dest, char * var, int n)
{
	memset(dest, 0, n);
	char * r = getenv(var);
	if (r)
	{
		strncpy(dest, r, n-1);
	}
}


void Cgi::SetEnv(char * src, char * var)
{
	if (var[0] == 0) return;

	char buffer[1128];
	sprintf(buffer, "%s=%s", var, src);
	_putenv(buffer);
}


// read all environment variables
void Cgi::ReadEnv()
{
	GetEnv(m_env.m_gatewayInterface, "GATEWAY_INTERFACE", Env::e_gatewayInterfaceSize);
	GetEnv(m_env.m_serverName, "SERVER_NAME", Env::e_serverNameSize);
	GetEnv(m_env.m_serverPort, "SERVER_PORT", Env::e_serverPortSize);
	GetEnv(m_env.m_serverProtocol, "SERVER_PROTOCOL", Env::e_serverProtocolSize);
	GetEnv(m_env.m_serverSoftware, "SERVER_SOFTWARE", Env::e_serverSoftwareSize);
	GetEnv(m_env.m_authType, "AUTH_TYPE", Env::e_authTypeSize);
	GetEnv(m_env.m_contentLength, "CONTENT_LENGTH", Env.e_contentLengthSize);
	GetEnv(m_env.m_contentType, "CONTENT_TYPE", Env.e_contentTypeSize);
	GetEnv(m_env.m_contentFile, "CONTENT_FILE", Env::e_contentFileSize);
	GetEnv(m_env.m_outputFile, "OUTPUT_FILE", Env::e_outputFileSize);
	GetEnv(m_env.m_pathInfo, "PATH_INFO", Env::e_pathInfoSize);
	GetEnv(m_env.m_pathTranslated, "PATH_TRANSLATED", Env::e_pathTranslatedSize);
	GetEnv(m_env.m_queryString, "QUERY_STRING", Env::e_queryStringSize);
	GetEnv(m_env.m_remoteAddr, "REMOTE_ADDR", Env::e_remoteAddrSize);
	GetEnv(m_env.m_remoteUser, "REMOTE_USER", Env::e_remoteUserSize);
	GetEnv(m_env.m_requestLine, "REQUEST_LINE", Env::e_requestLineSize);
	GetEnv(m_env.m_requestMethod, "REQUEST_METHOD", Env::e_requestMethodSize);
	GetEnv(m_env.m_scriptName, "SCRIPT_NAME", Env::e_scriptNameSize);
	GetEnv(m_env.m_accept, "HTTP_ACCEPT", Env::e_acceptSize);
	GetEnv(m_env.m_referrer, "HTTP_REFERRER", Env::e_referrerSize);
	GetEnv(m_env.m_userAgent, "HTTP_USER_AGENT", Env::e_userAgentSize);
	GetEnv(m_env.m_cookie, "HTTP_COOKIE", Env::e_cookieSize);
	GetEnv(m_env.m_httpFrom, "HTTP_FROM", Env::e_httpFromSize);
}


void Cgi::WriteEnv()
{
	SetEnv(m_env.m_gatewayInterface, "GATEWAY_INTERFACE");
	SetEnv(m_env.m_serverName, "SERVER_NAME");
	SetEnv(m_env.m_serverPort, "SERVER_PORT");
	SetEnv(m_env.m_serverProtocol, "SERVER_PROTOCOL");
	SetEnv(m_env.m_serverSoftware, "SERVER_SOFTWARE");
	SetEnv(m_env.m_authType, "AUTH_TYPE");
	SetEnv(m_env.m_contentLength, "CONTENT_LENGTH");
	SetEnv(m_env.m_contentType, "CONTENT_TYPE");
	SetEnv(m_env.m_contentFile, "CONTENT_FILE");
	SetEnv(m_env.m_outputFile, "OUTPUT_FILE");
	SetEnv(m_env.m_pathInfo, "PATH_INFO");
	SetEnv(m_env.m_pathTranslated, "PATH_TRANSLATED");
	SetEnv(m_env.m_queryString, "QUERY_STRING");
	SetEnv(m_env.m_remoteAddr, "REMOTE_ADDR");
	SetEnv(m_env.m_remoteUser, "REMOTE_USER");
	SetEnv(m_env.m_requestLine, "REQUEST_LINE");
	SetEnv(m_env.m_requestMethod, "REQUEST_METHOD");
	SetEnv(m_env.m_scriptName, "SCRIPT_NAME");
	SetEnv(m_env.m_accept, "ACCEPT");
	SetEnv(m_env.m_referrer, "REFERRER");
	SetEnv(m_env.m_userAgent, "USER_AGENT");
	SetEnv(m_env.m_cookie, "HTTP_COOKIE");
}


// read input from stdin
void Cgi::ReadStdin()
{
	m_contentLength = 0;
	if (m_env.m_contentLength[0] != 0)
		m_contentLength = atoi(m_env.m_contentLength);

	if (m_env.m_query) delete [] m_env.m_query;
	m_env.m_query = new char[m_contentLength+1];
	if (m_env.m_query == NULL)
	{
		CeresError e;
		throw(e);
	}

	memset(m_env.m_query, 0, m_contentLength+1);
	for (int i = 0; i < m_contentLength; i++)
	{
		int x = fgetc(stdin);
		m_env.m_query[i] = (char)x;
	}
}


// InitEnv
// This function is called first.  It reads all the cgi relevant 
// environment variables and performs whatever additional initialization
// is necessary.
void Cgi::InitEnv()
{
	ReadEnv();

	// other initialization can go here
}


void Cgi::InitQuery()
{
	// make sure there is a request method
	if (m_env.m_requestMethod[0] == 0)
	{
	}

	// request method is GET
	else if (stricmp(m_env.m_requestMethod, "GET")==0)
	{
		// delete any old query
		if (m_env.m_query) delete [] m_env.m_query;
		m_env.m_query = 0;

		// save the query string if available
		if (m_env.m_queryString)
		{
			int len = strlen(m_env.m_queryString)+1;
			m_env.m_query = new char[len];
			memset(m_env.m_query, 0, len);
			strcpy(m_env.m_query, m_env.m_queryString);
		}
	}

	// request method is POST
	else if (stricmp(m_env.m_requestMethod, "POST")==0)
	{
		ReadStdin();
	}

	// unknown request method
	else
	{
	}
}


// create a list of query name/value pairs
void Cgi::ParseQuery()
{
	// don't bother parsing if nothing is there
	if (!m_env.m_query) return;

	Normalize(m_env.m_query);
	m_queryList.CreateList(m_env.m_query, "&");
	m_queryList.SetValues("=");
}


// create a list of cookie name/value pairs
void Cgi::ParseCookie()
{
	Normalize(m_env.m_cookie);
	m_cookieList.CreateList(m_env.m_cookie, ";");
	m_cookieList.SetValues("=");
}


// calls all the init functions in the proper order
void Cgi::Init()
{
	InitEnv();
	InitQuery();
	ParseQuery();
	ParseCookie();
}


// ****************************************************************
// Printing Routines
// ****************************************************************

void Cgi::PrintCookie(char * name, char * value, 
					  char * path, char * domain, char * date)
{
	if (!name) return;
	if (!value) return;

	cout	<< "Set-Cookie: "
			<< name << "=" << value;

	if (!path) return;
	cout << "; path=" << path;

	if (!domain) return;
	cout << "; domain=" << domain;

	if (!date) return;
	cout << "; date=" << date;

	cout << endl;
}


void Cgi::PrintContentTypeHeader(const char * type)
{
	cout << "Content-Type: " << type << endl;
}


void Cgi::PrintLocationHeader(const char * location)
{
	printf("Location: %s\n\n", location);
}


// virtual function that can be
// overridden to create proper http header
// default is content type set to text/html
void Cgi::PrintHttpHeader()
{
	PrintContentTypeHeader("text/html");
}


void Cgi::PrintHtmlHeader()
{
	cout <<	"<html>\n";
	cout << "<head><title>Default Header</title></head>\n";
	cout << "<body>\n";
}


void Cgi::PrintHtmlTail()
{
	cout << "</body>\n";
	cout << "</html>\n";
}


// virtual function to be overridden
// by subclass to provide cgi functionality.
void Cgi::PrintBody()
{
	cout << "<H1>Success!!</H1>\n";
}


void Cgi::PrintQuery()
{
	cout << m_env.m_query << endl;
}


void Cgi::Run()
{
	PrintHttpHeader();
	PrintHtmlHeader();
	PrintBody();
	PrintHtmlTail();
}


// ****************************************************************
// Utility functions
// ****************************************************************


// convert hex to decimal
int Cgi::Hex2Dec(char c)
{
	if (c >= '0' && c <= '9')
		return (c - '0');

	if (c >= 'A' && c <= 'F')
		return (c - 'A' + 10);

	if (c >= 'a' && c <= 'f')
		return (c - 'a' + 10);

	return 0;
}


// copy source to dest replacing hex numbers with decimal numbers
void Cgi::ReplaceHex(char * dest, char * source, size_t & i, size_t & j)
{
	int num;

	// skip #
	i++;

	// convert first digit
	if (i < strlen(source))
	{
		num = Hex2Dec(source[i]) * 16;
	}

	// move to next digit
	i++;

	// convert next digit
	if (i < strlen(source))
	{
		num = num + Hex2Dec(source[i]);
	}

	// put character into input
	dest[j] = (char) num;
}


// replace special characters and hex numbers
void Cgi::Normalize(char * input)
{
	char tmpInput[1024];
	strcpy(tmpInput, input);

	size_t i, j;
	for (i = 0, j = 0; i < strlen(input); i++, j++)
	{
		if (tmpInput[i] == '+')
			input[j] = ' ';
		else if (tmpInput[i] == '%')
			ReplaceHex(input, tmpInput, i, j);
		else
			input[j] = tmpInput[i];
	}
}


// ****************************************************************
// Get functions
// ****************************************************************

// return a query value given its name
char * Cgi::GetQueryValue(char * name)
{
	for (int i = 0; i < m_queryList.Size(); i++)
	{
		Pair * p = m_queryList[i];
		if (strcmp(p->m_name, name) == 0)
		{
			return p->m_value;
		}
	}

	return 0;
}


// return a cookie value given its name
char * Cgi::GetCookieValue(char * name)
{
	for (int i = 0; i < m_cookieList.Size(); i++)
	{
		Pair * p = m_cookieList[i];
		if (strcmp(p->m_name, name) == 0)
		{
			return p->m_value;
		}
	}

	return 0;
}


// return entire query
char * Cgi::GetQuery()
{
	return m_env.m_query;
}


// return entire cookie
char * Cgi::GetCookie()
{
	return m_env.m_cookie;
}


Env & Cgi::GetEnv()
{
	return m_env;
}
