// cgi.h
// copyright (C) 2000 by David Cox All Rights Reserved

#ifndef __CGI_H
#define __CGI_H

#include <stddef.h>
#include "pair.h"

struct Env
{
	enum {
		e_gatewayInterfaceSize = 32,
		e_serverNameSize = 32,
		e_serverPortSize = 8,
		e_serverProtocolSize = 16,
		e_serverSoftwareSize = 32,

		e_authTypeSize = 16,
		e_contentFileSize = 128,
		e_contentLengthSize = 8,
		e_contentTypeSize = 16,
		e_outputFileSize = 128,
		e_pathInfoSize = 128,
		e_pathTranslatedSize = 128,
		e_queryStringSize = 256,
		e_remoteAddrSize = 128,
		e_remoteUserSize = 16,
		e_requestLineSize = 128,
		e_requestMethodSize = 16,
		e_scriptNameSize = 32,
		e_httpFromSize = 64,

		e_acceptSize = 64,
		e_referrerSize = 128,
		e_userAgentSize = 64,
		e_cookieSize = 256,
		e_querySize = 1024,
		e_dirSize = 512
	};

	char m_gatewayInterface[e_gatewayInterfaceSize];
	char m_serverName[e_serverNameSize];
	char m_serverPort[e_serverPortSize];
	char m_serverProtocol[e_serverProtocolSize];
	char m_serverSoftware[e_serverSoftwareSize];

	char m_authType[e_authTypeSize];
	char m_contentFile[e_contentFileSize];
	char m_contentLength[e_contentLengthSize];
	char m_contentType[e_contentTypeSize];
	char m_outputFile[e_outputFileSize];
	char m_pathInfo[e_pathInfoSize];
	char m_pathTranslated[e_pathTranslatedSize];
	char m_queryString[e_queryStringSize];
	char m_remoteAddr[e_remoteAddrSize];
	char m_remoteUser[e_remoteUserSize];
	char m_requestLine[e_requestLineSize];
	char m_requestMethod[e_requestMethodSize];
	char m_scriptName[e_scriptNameSize];
	char m_httpFrom[e_httpFromSize];

	char m_accept[e_acceptSize];
	char m_referrer[e_referrerSize];
	char m_userAgent[e_userAgentSize];
	char m_cookie[e_cookieSize];
	char m_dir[e_dirSize];
	char * m_query;

	Env();
	~Env();

	void Print();
};

class Cgi
{
public:

	Cgi();
	~Cgi();

	void PrintContentTypeHeader(const char *);
	void PrintLocationHeader(const char *);

	void InitEnv();
	void InitQuery();
	void PrintQuery();
	void ParseQuery();
	void ParseCookie();
	void Init();

	char * GetQueryValue(char *);
	char * GetCookieValue(char *);
	char * GetQuery();
	char * GetCookie();
	Env  & GetEnv();

	void PrintCookie(char * name, char * value, 
		char * path, char * domain, char * date);

	virtual void PrintHttpHeader();
	virtual void PrintHtmlHeader();
	virtual void PrintHtmlTail();
	virtual void PrintBody();
	virtual void Run();

	void	Normalize(char *);

private:

	char		m_hexToken;
	int			m_contentLength;
	Env			m_env;
	PairList	m_queryList;
	PairList	m_cookieList;

	void	GetEnv(char * dest, char * var, int n);
	void	SetEnv(char * dest, char * var);
	void	ReadEnv();
	void	WriteEnv();
	void	ReadStdin();
	void	CreateList(PairList &, char *, char *);
	void	PrintList(PairList &);
	void	SetValues(PairList &, char *);
	void	ReplaceHex(char * dest, char * source, size_t &, size_t &);
	int		Hex2Dec(char);
};


#endif
