// Database.cpp
// copyright (C) 2000 by David Cox All Rights Reserved

#include <windows.h>
#include "Database.h"
#include "cereserror.h"
#include <stdio.h>
#include <string.h>
#include <iostream.h>



// ***************************************************
// Column Class


Column::Column()
{
	memset(m_name, 0, e_nameSize);
	m_dataSzInd = 0;
	m_data = 0;
	m_pageSize = 0;
}


Column::Column(const Column & c)
{
	// allocate data buffers
	m_pageSize = c.m_pageSize;
	m_data = new char[m_pageSize * e_dataSize];

	if (m_data == 0)
	{
		CeresError e;
		e.Msg("Cannot allocate memory for column data");
		e.Line(__LINE__);
		e.File(__FILE__);
		throw (e);
	}
	memset(m_data, 0, m_pageSize * e_dataSize);

	m_dataSzInd = new SQLINTEGER[m_pageSize];
	if (m_dataSzInd == 0)
	{
		delete [] m_data;
		CeresError e;
		e.Msg("Cannot allocate memory for column dataSzInd");
		e.Line(__LINE__);
		e.File(__FILE__);
		throw (e);
	}
	memset(m_dataSzInd, 0, m_pageSize);

	// copy buffers
	memcpy(m_data, c.m_data, m_pageSize * e_dataSize);
	memcpy(m_dataSzInd, c.m_dataSzInd, m_pageSize);
	memcpy(m_name, c.m_name, e_nameSize);
}


Column::~Column()
{
	delete [] m_data;
	delete [] m_dataSzInd;
}


Column & Column::operator=(const Column & c)
{
	if (this == &c) return *this;

	// save page size
	m_pageSize = c.m_pageSize;

	// reallocate data buffers
	if (m_data) delete [] m_data;
	m_data = new char[m_pageSize * e_dataSize];
	if (m_data == 0)
	{
		CeresError e;
		e.Msg("Cannot allocate memory for column data");
		e.Line(__LINE__);
		e.File(__FILE__);
		throw (e);
	}
	memset(m_data, 0, m_pageSize * e_dataSize);

	if (m_dataSzInd) delete [] m_dataSzInd;
	m_dataSzInd = new SQLINTEGER[m_pageSize];
	if (m_dataSzInd == 0)
	{
		delete [] m_data;
		CeresError e;
		e.Msg("Cannot allocate memory for column dataSzInd");
		e.Line(__LINE__);
		e.File(__FILE__);
		throw (e);
	}
	memset(m_dataSzInd, 0, m_pageSize);

	// copy buffers
	memcpy(m_data, c.m_data, m_pageSize * e_dataSize);
	memcpy(m_dataSzInd, c.m_dataSzInd, m_pageSize);
	memcpy(m_name, c.m_name, e_nameSize);

	return *this;
}


void Column::PageSize(int pagesize)
{
	// save page size
	m_pageSize = pagesize;

	// reallocate data buffers
	if (m_data) delete [] m_data;
	m_data = new char[m_pageSize * e_dataSize];
	if (m_data == 0)
	{
		CeresError e;
		e.Msg("Cannot allocate memory for column data");
		e.Line(__LINE__);
		e.File(__FILE__);
		throw (e);
	}
	memset(m_data, 0, m_pageSize * e_dataSize);

	if (m_dataSzInd) delete [] m_dataSzInd;
	m_dataSzInd = new SQLINTEGER[m_pageSize];
	if (m_dataSzInd == 0)
	{
		delete [] m_data;
		CeresError e;
		e.Msg("Cannot allocate memory for column dataSzInd");
		e.Line(__LINE__);
		e.File(__FILE__);
		throw (e);
	}
	memset(m_dataSzInd, 0, m_pageSize);
}


void Column::Name(const char * name)
{
	memset(m_name, 0, e_nameSize);
	strncpy(m_name, name, e_nameSize-1);
	
}


char * Column::Name()
{
	return m_name;
}


void Column::Data(int row, const char * data)
{
	memset(&m_data[row * e_dataSize], 0, e_dataSize);
	memcpy(&m_data[row * e_dataSize], data, e_dataSize-1);
}


char * Column::Data(int row)
{
	return &m_data[row * e_dataSize];
}


char * Column::Data()
{
	return m_data;
}


SQLINTEGER * Column::DataSzInd()
{
	return m_dataSzInd;
}



// ***************************************************
// Table Class

Table::Table(int pagesize)
{
	m_cols = 0;
	m_pageSize = pagesize;
	m_numCols = 0;
	m_numRows = 0;
}


Table::Table(const Table & t)
{
	// copy basic info
	m_pageSize = t.m_pageSize;
	m_numCols = t.m_numCols;
	m_numRows = t.m_numRows;
	m_cols = 0;

	// allocate memory for columns
	if (m_numCols > 0)
	{
		m_cols = new Column[m_numCols];

		if (m_cols == 0)
		{
			CeresError e;
			e.Msg("Cannot allocate memory for columns");
			e.Line(__LINE__);
			e.File(__FILE__);
		}
	}
		
	// copy columns
	for (int i = 0; i < m_numCols; i++)
	{
		m_cols[i] = t.m_cols[i];
	}
}


Table::~Table()
{
	if (m_cols) delete [] m_cols;
}


Table & Table::operator=(const Table & t)
{
	if (this == &t) return *this;

	// copy basic info
	m_pageSize = t.m_pageSize;
	m_numCols = t.m_numCols;
	m_numRows = t.m_numRows;
	m_cols = 0;

	// allocate memory for columns
	if (m_numCols > 0)
	{
		m_cols = new Column[m_numCols];

		if (m_cols == 0)
		{
			CeresError e;
			e.Msg("Cannot allocate memory for columns");
			e.Line(__LINE__);
			e.File(__FILE__);
		}
	}
		
	// copy columns
	for (int i = 0; i < m_numCols; i++)
	{
		m_cols[i] = t.m_cols[i];
	}

	return *this;
}


void Table::NumCols(int numCols)
{
	if (m_cols) delete [] m_cols;

	if (numCols > 0)
	{
		m_numCols = numCols;
		m_cols = new Column[numCols];
	}
	else
	{
		m_numCols = 0;
		m_cols = 0;
	}
		
	if (m_cols != 0)
	{
		for (int i = 0; i < m_numCols; i++)
			m_cols[i].PageSize(m_pageSize);
	}
}


int Table::NumCols()
{
	return m_numCols;
}


void Table::NumRows(int r)
{
	m_numRows = r;
}


int Table::NumRows()
{
	return m_numRows;
}


int Table::PageSize()
{
	return m_pageSize;
}


Column & Table::GetCol(int i)
{
	return m_cols[i];
}


void Table::ColName(int i, const char * colName)
{
	m_cols[i].Name(colName);
}


char * Table::Value(int row, int col)
{
	return m_cols[col].Data(row);
}



// ***************************************************
// Database Class


Database::Database()
{
	m_pageSize = 0;
	m_henv = 0;
	m_hdbc = 0;
	m_hstmt = 0;
	m_retcode = 0;
	m_numResultCols = 0;
	m_rowsFetched = 0;
	memset(m_datasource, 0, e_datasourceSize);
	memset(m_user, 0, e_userSize);
	memset(m_password, 0, e_passwordSize);
}


Database::~Database()
{
}


void Database::Datasource(const char * s)
{
	memset(m_datasource, 0, e_datasourceSize);
	memcpy(m_datasource, s, e_datasourceSize-1);
}


const char * Database::Datasource()
{
	return m_datasource;
}


void Database::User(const char * s)
{
	memset(m_user, 0, e_userSize);
	memcpy(m_user, s, e_userSize-1);
}


const char * Database::User()
{
	return m_user;
}


void Database::Password(const char * s)
{
	memset(m_password, 0, e_passwordSize);
	memcpy(m_password, s, e_passwordSize-1);
}


const char * Database::Password()
{
	return m_password;
}


void Database::OpenDatabase()
{
	// allocate environment handle
	m_retcode = SQLAllocEnv(&m_henv); 
	if (m_retcode != SQL_SUCCESS)
	{
		CeresError e;
		e.Line(__LINE__);
		e.File(__FILE__);
		throw (e);
	}

	// allocate connection handle
	m_retcode = SQLAllocConnect(m_henv, &m_hdbc);
	if ((m_retcode != SQL_SUCCESS) &&
		(m_retcode != SQL_SUCCESS_WITH_INFO))
	{
		SQLFreeEnv(m_henv);

		CeresError e;
		e.Line(__LINE__);
		e.File(__FILE__);
		throw (e);
	}

	/* Set login timeout to 5 seconds. */
	SQLSetConnectOption(m_hdbc, SQL_LOGIN_TIMEOUT, 5);

	/* Connect to data source */
	m_retcode = SQLConnect(m_hdbc, 
		(unsigned char *)m_datasource, SQL_NTS, 
		(unsigned char *)m_user, SQL_NTS, 
		(unsigned char *)m_password, SQL_NTS);

	if ((m_retcode != SQL_SUCCESS) && 
		(m_retcode != SQL_SUCCESS_WITH_INFO))
	{
		unsigned char buffer[CeresError::e_msgSize];
		SQLSMALLINT		RecNumber = 1;
		SQLCHAR			Sqlstate;
		SQLINTEGER		NativeError;
		SQLSMALLINT		TextLength;

		m_retcode = SQLGetDiagRec(
			SQL_HANDLE_DBC,
			m_hdbc,
			RecNumber,
			&Sqlstate,
			&NativeError,
			buffer,
			CeresError::e_msgSize-1,
			&TextLength);

		SQLFreeConnect(m_hdbc);
		SQLFreeEnv(m_henv);

		CeresError e;
		e.Line(__LINE__);
		e.File(__FILE__);
		e.Msg((const char *)buffer);
		throw (e);
	}

	// allocate statement handle
	m_retcode = SQLAllocStmt(m_hdbc, &m_hstmt); 
	if (m_retcode != SQL_SUCCESS) 
	{
		SQLDisconnect(m_hdbc);
		SQLFreeConnect(m_hdbc);
		SQLFreeEnv(m_henv);
		SQLFreeStmt(m_hstmt, SQL_DROP);            

		CeresError e;
		e.Line(__LINE__);
		e.File(__FILE__);
		throw (e);
	}
}


void Database::CloseDatabase()
{
	SQLDisconnect(m_hdbc);
	SQLFreeConnect(m_hdbc);
	SQLFreeEnv(m_henv);
}


int Database::RowsFetched()			
{
	return m_rowsFetched;
}


int Database::NumResultCols()
{
	m_numResultCols = 0;
	SQLNumResultCols(m_hstmt, &m_numResultCols);
	return m_numResultCols;
}


void Database::VerifyFetch()
{
	if ((m_retcode == SQL_SUCCESS) || 
		(m_retcode == SQL_SUCCESS_WITH_INFO))
	{
	}

	else if (m_retcode == SQL_INVALID_HANDLE)
	{
		CeresError e;
		e.Msg("Invalid handle");
		e.Line(__LINE__);
		e.File(__FILE__);
		throw (e);
	}

	else if (m_retcode == SQL_STILL_EXECUTING)
	{
		CeresError e;
		e.Line(__LINE__);
		e.File(__FILE__);
		throw (e);
	}

	else if (m_retcode == SQL_NO_DATA_FOUND)
	{
		CeresError e;
		e.Msg("No data found");
		e.Line(__LINE__);
		e.File(__FILE__);
		throw (e);
	}

	else
	{
		CeresError e;
		e.Msg("Unknown Error");
		e.Line(__LINE__);
		e.File(__FILE__);
		throw (e);
	}
}


void Database::VerifySQLExecution()
{
	SQLSMALLINT		RecNumber = 1;
	SQLCHAR			Sqlstate;
	SQLINTEGER		NativeError;
	SQLSMALLINT		TextLength;
	unsigned char msg[CeresError::e_msgSize];
	memset(msg, 0, sizeof(msg));

	if (m_retcode != SQL_SUCCESS)
	{
		m_retcode = SQLGetDiagRec(
			SQL_HANDLE_STMT,
			m_hstmt,
			RecNumber,
			&Sqlstate,
			&NativeError,
			msg,
			sizeof(msg)-1,
			&TextLength);

		CeresError e;
		e.Msg((const char *)msg);
		e.Line(__LINE__);
		e.File(__FILE__);
		throw (e);
	}
}


void Database::Select(const char * sql, int page, Table & table)
{
	// init some variables
	m_pageSize = table.PageSize();
	UWORD * rowstatus = new UWORD[m_pageSize];
	SQLUINTEGER rowsFetched = 0;
	m_rowsFetched = 0;

	// create sql statement
	char stmt[1600];
	char * tstmt = "%s";
	sprintf(stmt, tstmt, sql);

	// set statment options
	m_retcode = SQLSetStmtOption(m_hstmt, SQL_ROWSET_SIZE, m_pageSize);
	m_retcode = SQLSetStmtOption(m_hstmt, SQL_CURSOR_TYPE, SQL_CURSOR_STATIC);

	// execute sql
	m_retcode = SQLExecDirect(m_hstmt, (unsigned char *)stmt, strlen((const char*)stmt));
	VerifySQLExecution();

	// get the number of columns and construct table
	short numCols;
	m_retcode = SQLNumResultCols(m_hstmt, &numCols);
	table.NumCols(numCols);

	// get the column names and bind column data field
	for (int n = 1; n <= numCols; n++)
	{
		UCHAR colName[50];
		SWORD cbColName;
		SWORD colType;
		UDWORD cbColDef;
		SWORD colScale;
		SWORD colNullable;

		// get some useful info about the column
		SQLDescribeCol(m_hstmt, n, colName, sizeof(colName), 
			&cbColName, &colType, &cbColDef, &colScale, &colNullable);

		// save the name of the column
		table.ColName(n-1, (const char *)colName);
		
		// bind the column to the buffers
		Column & col = table.GetCol(n-1);
		SQLBindCol(m_hstmt, n, SQL_C_CHAR, 
			col.Data(), col.e_dataSize, col.DataSzInd());
	}

	// fetch page of data
	int row = page * m_pageSize + 1;
	m_retcode = SQLExtendedFetch(m_hstmt, 
		SQL_FETCH_ABSOLUTE, row, &m_rowsFetched, rowstatus);

	table.NumRows(m_rowsFetched);
	VerifyFetch();
}


