// Database.h
// copyright (C) 2000 by David Cox All Rights Reserved

#ifndef __Database_H
#define __Database_H

#include <sql.h>
#include <sqlext.h>


class Column
{
public:

	enum {
		e_nameSize = 50,
		e_rowsetSize = 10,
		e_dataSize = 128
	};

	Column();
	Column(const Column &);
	~Column();
	Column & operator=(const Column &);

	void Name(const char *);
	char * Name();

	void Data(int row, const char *);
	char * Data(int row);
	char * Data();
	SQLINTEGER * DataSzInd();

	void PageSize(int);

private:

	char m_name[e_nameSize];
	char * m_data;
	SQLINTEGER * m_dataSzInd;
	int m_pageSize;
};


class Table
{
public:

	Table(int pagesize);
	Table(const Table &);
	~Table();
	Table &operator=(const Table &);

	void NumCols(int);
	int  NumCols();
	void NumRows(int);
	int  NumRows();
	int  PageSize();

	void ColName(int col, const char * name);
	Column & GetCol(int col);
	char * Value(int row, int col);

private:

	Column * m_cols;
	int m_numCols;
	int m_numRows;
	int m_pageSize;
};


class Database
{
public:

	enum {
		e_datasourceSize = 64,
		e_userSize = 32,
		e_passwordSize = 32
	};

	Database();
	Database(const Database &);
	~Database();
	Database &operator=(const Database &);

	void OpenDatabase();
	void CloseDatabase();
	SQLHDBC GetConnectionHandle();

	int		RowsFetched();
	int		NumResultCols();
	void	Select(const char * sql, int page, Table & table);

	void			Datasource(const char *);
	const char *	Datasource();

	void			User(const char *);
	const char *	User();

	void			Password(const char *);
	const char *	Password();

private:


	HENV    m_henv;
	HDBC    m_hdbc;
	HSTMT   m_hstmt;
	RETCODE m_retcode;

	char m_datasource[e_datasourceSize];
	char m_user[e_userSize];
	char m_password[e_passwordSize];

	SWORD		m_numResultCols;
	UDWORD		m_rowsFetched;
	int			m_pageSize;

	void VerifySQLExecution();
	void VerifyFetch();
};


#endif
