package keepalive.apps;

import java.awt.*;
import java.awt.event.*;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FilenameFilter;
import java.io.IOException;
import java.lang.reflect.Method;

/**
 * An app for launching the various apps: SimApplication, the Socket client,
 * and the Socket Server.
 *
 * @author Joshua Fox
 */
public class RunApp extends Frame{
    private static String workingDir =".";
    private String runThis; // class name whose main() we'll run
    private String protocolPropFile;
    private boolean disposeOnRun = false;
    private boolean noAppHasBeenRun = true;

    // indices to apps array:
    private static final int SIM_APPLICATION = 0;
    private static final int CLIENT = 1;
    private static final int SERVER = 2;
    private static final String apps [][] =
                 {
                   {"SimApplication", "keepalive.apps.SimApplication"},
                   {"Client (with TCP/IP sockets)", "keepalive.apps.Client"},
                   {"Server (with TCP/IP sockets)", "keepalive.apps.Server"}
                 };


    private static final String  chooseApplication = "Choose Application";
    private static final String  chooseProtocol = "Choose Protocol";

    GridLayout gridLayoutForRunPanel = new GridLayout();
    GridLayout gridLayoutForRunApp = new GridLayout();

    BorderLayout borderLayout1 = new BorderLayout();
    BorderLayout borderLayout2 = new BorderLayout();
    BorderLayout borderLayout3 = new BorderLayout();
    BorderLayout borderLayout4 = new BorderLayout();
    BorderLayout borderLayout5 = new BorderLayout();

    Panel chooseAppPanel = new Panel();
    Panel protocolPanel = new Panel();
    Panel protocolLabelPanel = new Panel();
    Panel protocolChoicePanel = new Panel();
    Panel serverAddressPanel = new Panel();
    Panel runPanel = new Panel();

    Label protocolExplanationLabel = new Label();
    Label chooseAppLabel = new Label();
    Label serverAddressLabel = new Label();
    Label chooseProtocolLabel = new Label();

    TextField addressTextField = new TextField();

    Choice appChoice = new Choice();
    Choice protocolChoice = new Choice();

    Button runButton = new Button();
    Button protocolDetailsButton = new Button();

    public RunApp() {
        initialize();

        appChoice.addItem(chooseApplication);
        for (int i=0 ; i < apps.length ; i++) {
            appChoice.addItem(apps[i][0]);
        }

        this.setLocation(100,50);
        this.setVisible(true);
    }

    private void initialize() {
        this.setSize(new Dimension(448, 490));
        this.setTitle("Run demos");

        // layouts
        this.setLayout(gridLayoutForRunApp);
        gridLayoutForRunApp.setRows(3);
        gridLayoutForRunApp.setColumns(1);
        serverAddressPanel.setLayout(borderLayout1);
        runPanel.setLayout(gridLayoutForRunPanel);
        protocolPanel.setLayout(borderLayout2);
        chooseAppPanel.setLayout(borderLayout3);
        gridLayoutForRunPanel.setColumns(1);
        gridLayoutForRunPanel.setRows(2);
        protocolLabelPanel.setLayout(borderLayout4);
        protocolChoicePanel.setLayout(borderLayout5);

        // labels, text, color
        chooseAppLabel.setText(chooseApplication);
        runButton.setFont(new Font("Dialog", 0, 24));
        protocolDetailsButton.setFont(new Font("Dialog", 0, 24));
        runButton.setLabel("Run");
        addressTextField.setText("localhost");
        serverAddressLabel.setText("Server Address");
        chooseProtocolLabel.setText(chooseProtocol);
        protocolDetailsButton.setLabel("Details");
        addressTextField.setForeground(Color.lightGray);

        // listeners
        protocolDetailsButton.addActionListener(
                                           new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                protocolDetailsButton_actionPerformed(e);
            }
        });

        protocolChoice.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(ItemEvent ie) {
                protocolChoice_itemStateChanged(ie);
            }
        });

        this.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(WindowEvent we) {
                this_windowClosing(we);
            }
        });

        appChoice.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(ItemEvent ie) {
                appChoice_itemStateChanged(ie);
            }
        });


        runButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                runButton_actionPerformed(ev);
            }
        });


        // chooseAppPanel
        this.add(chooseAppPanel, null);
        chooseAppPanel.add(chooseAppLabel, BorderLayout.NORTH);
        chooseAppPanel.add(appChoice, BorderLayout.CENTER);

        // chooseProtocolPanel
        this.add(protocolPanel, null);
        protocolPanel.add(protocolLabelPanel, BorderLayout.NORTH);
        protocolLabelPanel.add(chooseProtocolLabel, BorderLayout.NORTH);
        protocolLabelPanel.add(protocolExplanationLabel, BorderLayout.CENTER);
        protocolPanel.add(serverAddressPanel, BorderLayout.SOUTH);

        // serverAddressPanel within chooseProtocolPanel
        serverAddressPanel.add(addressTextField, BorderLayout.CENTER);
        serverAddressPanel.add(serverAddressLabel, BorderLayout.EAST);
        protocolPanel.add(protocolChoicePanel, BorderLayout.CENTER);
        protocolChoicePanel.add(protocolChoice, BorderLayout.CENTER);

        // runPanel
        this.add(runPanel, null);
        runPanel.add(protocolDetailsButton, null);
        runPanel.add(runButton, null);

        // disabling
        runButton.setEnabled(false);
        protocolDetailsButton.setEnabled(false);
        protocolPanel.setEnabled(false);
        serverAddressPanel.setEnabled(false);

    }


    void runButton_actionPerformed(ActionEvent ae) {
        noAppHasBeenRun = false;
        new MainThread().start();
        if (disposeOnRun) {
            dispose();
        }
    }

    void appChoice_itemStateChanged(ItemEvent ie) {
        String itemSelected = (String) ie.getItem();
        if (itemSelected.equals(chooseApplication)) {
            return;
        }

        protocolPanel.setEnabled(true);

        for (int i=0; i< apps.length; i++) {
            if (itemSelected.equals(apps[i][0])) {
                runThis = apps[i][1];
            }
        }

        addKeepAliveProtocolChoice();

        // With SimApplication, we may close this Frame after running app.
        // With socket Client/Server, we explain that protocols need to match
        if (itemSelected.equals(apps[SIM_APPLICATION][0])) {
            showExplanationLabel(false);
            disposeOnRun=true;
        } else {
            showExplanationLabel(true);
            disposeOnRun=false;
        }

        // Only Socket Client app needs to know server address
        if (itemSelected.equals(apps[CLIENT][0])) {
            serverAddressPanel.setEnabled(true);
            addressTextField.setForeground(Color.black);
        } else {
             serverAddressPanel.setEnabled(false);
             addressTextField.setForeground(Color.lightGray);
        }
    }


    void protocolChoice_itemStateChanged(ItemEvent ie) {
        String itemSelected = (String)ie.getItem();
        if (itemSelected.equals(chooseProtocol)) {
            return;
        }

        protocolPropFile = itemSelected;
        runButton.setEnabled(true);
        protocolDetailsButton.setEnabled(true);

    }

    void this_windowClosing(WindowEvent e) {
       dispose();
       if (noAppHasBeenRun) {
           System.exit(0);
       }
    }

    private void showExplanationLabel(boolean show) {
        protocolExplanationLabel.setText(
            show?"(Server & Client should choose the same protocol)":"");
    }

    private void addKeepAliveProtocolChoice() {
        File dir = new File(workingDir);
        String [] propFiles = dir.list(new DirFilter("properties"));
        protocolChoice.addItem(chooseProtocol);
        for (int i=1; i<propFiles.length; i++) {
            protocolChoice.addItem(propFiles[i]);
        }
    }

     public static void main(String [] args) {

        ArgReader.showUsageIfRequested(args, 
                                    "Usage: java RunApp [-d workingDirectory]");
        workingDir = ArgReader.getArgBySwitch(args, "d", workingDir);
        new RunApp();
     }


    void protocolDetailsButton_actionPerformed(ActionEvent e) {
        String protocolFile= protocolChoice.getSelectedItem();
        if (protocolFile.equals(chooseProtocol)) {
            return;
        }
        final Dialog dialog = new Dialog(this, "Protocol Details" , true);
        dialog.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {dialog.dispose();}
        });

        TextArea ta = new TextArea(loadFile(protocolFile));
        ta.setEditable(false);
        ta.setFont(new Font("Monospaced", Font.ITALIC, 12));

        dialog.add(ta, BorderLayout.CENTER);
        dialog.setLocation(this.getLocation().x - 10, this.getLocation().y - 10);
        dialog.setSize(500,300);
        dialog.setVisible(true);

    }


    private String loadFile(String fileName) {
        try {
            String returnValue = "";
            File file = new File(workingDir, fileName);
            BufferedReader reader = new BufferedReader(new FileReader(file));
            String s = null;
            while ( (s=reader.readLine()) !=null) {
                returnValue+=(s+"\n");
            }
            return returnValue;
        } catch (IOException ioe) {
            return "File " + workingDir + File.separator + 
                                                        fileName + " not found";
        }
    }
   
    private class DirFilter implements FilenameFilter {
        String suffix;
        DirFilter(String suffix) {
            this.suffix = suffix;
        }

        public boolean accept(File dir, String fullName) {
            String fileName = new File(fullName).getName();
            int dotIndex = fileName.lastIndexOf(".");
            return (dotIndex!=-1 && 
                   fileName.substring(dotIndex).equalsIgnoreCase("." + suffix));
        }
    }

    private class MainThread extends Thread {
        MainThread() {
            super("MainThread");
        }

        public  void run() {
            if (protocolPropFile==null || runThis ==null) return;

            try {
                Class [] stringArrayClassArray = 
                                        {Class.forName("[Ljava.lang.String;") };
                Class clazz = Class.forName(runThis);
                Method meth = clazz.getMethod("main" , stringArrayClassArray);

                // Actually, Server Address param only needed for 
                // socket-based client, but others will ignore it.
                String serverAddress = addressTextField.getText();
                String [] argArray = {"-d", workingDir, "-k", 
                                        protocolPropFile, "-s", serverAddress};
                Object [] argsForMethodInvoke = {argArray};
                meth.invoke(null, argsForMethodInvoke);

            } catch (Exception e) {
                System.err.println("Could not run application " + runThis + 
                                                                      "; " + e);
            }
        }
    }
}