package keepalive.apps;

import keepalive.socket.SocketFactory;
import keepalive.socket.sim.SimHost;

/**
 * Runs a client application and a server application in a simulated networking
 * environment.
 * The behavior of sockets on a network is simulated in a single VM.
 *
 * @author Joshua Fox
 *
 */
public class SimApplication {

    /**
     * Start the simulated client and server applications.
     *
     * @param args command line arguments; use -help to get usage description.
     */
    public static void main(String [] args) {
        ArgReader.showUsageIfRequested(args,
            "Usage: java SimApplication [-k protocolFile] [-d workingDir]\n");

        String workingDir = ArgReader.getArgBySwitch(args, "d", ".");
        String clientKeepAliveClassName =
            ArgReader.getPropertyBySwitch(args, workingDir,
                           "k", "client", "keepalive.keepalive.NullKeepAlive");
        SocketFactory.setClientKeepAlive(clientKeepAliveClassName);

        String serverKeepAliveClassName =
            ArgReader.getPropertyBySwitch(args, workingDir,
                            "k", "server", "keepalive.keepalive.NullKeepAlive");
        SocketFactory.setServerKeepAlive(serverKeepAliveClassName);

        SocketFactory.setSocketType(SocketFactory.SIM_SOCKET);

        // Simulated Client and Server machines, with comm plugs and netstats
        new SimHost(false);
        new SimHost(true);
    }
}
