package keepalive.communication;

import java.io.IOException;

/**
 * An object that knows how to send a message.
 */
public interface Sender {
    /**
     * Send the given String.
     *
     * @param s the String to send.
     * @exception IOException if an I/O error occurs.
     */
    public void send(String s) throws IOException;

    /**
     * Get the last time data was sent.
     *
     * @return when data was last sent, in milliseconds since the epoch. If
     *     data has not yet been sent, return the time of instantiation of this
     *     Object.
     */
    public long lastTimeStamp();

    /**
     * Closes socket without rethrowing IOException.
     *
     * The Keep-Alives close this Sender when they detect a communication
     * failure. Closing the Sender also closes the Receiver, so that the
     * entire connection to the client is cleaned up.
     *
     * This demo application just closes the socket; in your application
     * you might also close database connections devoted to servicing the
     * client, remove table entries referring to the client, etc.
     *
     */
    public void close();
}