package keepalive.keepalive;

import java.io.IOException;
import keepalive.communication.Sender;


/**
 * This ActiveKeepAlive repeatedly sends out dummy messages  to
 * see if we get an IOException.  (A more sophisticated version might
 * change the frequency of the outgoing messages according to measured
 * network parameters.
 * This Keep-Alive can be used with another ActiveKeepAlive,
 * with a PassiveKeepAlive, or with a NullKeepAlive on the remote side.
 *
 * @see keepalive.keepalive.PassiveKeepAlive
 * @see keepalive.keepalive.ActiveKeepAlive
 * @see keepalive.keepalive.NullKeepAlive
 *
 * @author Joshua Fox
 */
public class ActiveKeepAlive extends KeepAlive {

    /**
     *
     * @param sender
     */
    public ActiveKeepAlive(Sender sender) {
        super(null, sender);
    }

    /**
     * Used for dynamic classloading
     */
    public ActiveKeepAlive() {
    }

    /**
    */
    public void run() {
        while (!endRequested()) {
            try {
                getSender().send(getDummyString());
            } catch (IOException ioe) { // we've detected a connection failure
                getSender().close();
                break;
            }

            try {
                // Every now and then, wake up to check the situation.
                //
                // We could make this more sophisticated by piggybacking--
                // resetting the timer every time some data is sent instead of 
                // just sending at constant intervals. In other words, if our
                // application is sending data, then that is a sufficient
                // test of the sytem, there is no need for this Keep-Alive
                // thread to also send data.
                sleep(TIME_BETWEEN_ACTIVE_PROBES);
            } catch (InterruptedException ie) { }

       }
    }
}
