package keepalive.keepalive;

import java.io.IOException;

import keepalive.communication.Sender;
import keepalive.communication.Receiver;


/**
 * Combines the ActiveKeepAlive and AreYouThere.
 * Used with PassiveKeepAliveWithAYT
 * Serves as an example of combining Keep-Alive
 * protocols, as will usually happen in real programs.
 *
 * @see keepalive.keepalive.ActiveKeepAlive
 * @see keepalive.keepalive.AreYouThere
 * @see keepalive.keepalive.PassiveKeepAliveWithIHY
 *
 * @author Joshua Fox
 */
public class ActiveKeepAliveWithAYT extends KeepAlive {
    /**
     * Close the connection if no incoming message has
     * been received in this time.
     */
    private static final long maxPermittedIdleTime =
                                                6 * TIME_BETWEEN_ACTIVE_PROBES;

    /**
     * Send an Are-You-There if no incoming message has
     * been received in this time.
     */
    private static final long idleTimeBeforeAYT =
                                        (long)(0.67d * maxPermittedIdleTime);

    /**
     * Create a Keep-Alive object.
     */
    public ActiveKeepAliveWithAYT(Receiver receiver, Sender sender) {
        super(receiver, sender);
    }

    /**
     * Create a Keep-Alive object.
     * Used for dynamic classloading
     */
    public ActiveKeepAliveWithAYT() {
    }


    /**
     *
     */
    public void run() {
        while (!endRequested()) {
            try {
                getSender().send(getDummyString());
            } catch (IOException ioe) {
                getSender().close();
                break;
            }

            sendAYTIfIdleForTooLong();
            closeIfIdleForTooLong();

            try {
                sleep(TIME_BETWEEN_ACTIVE_PROBES);
            } catch (InterruptedException ie) {
            }
        }
    }


    /**
     *
     */
    private void closeIfIdleForTooLong() {
        if (getSilentTimeForIncoming() > maxPermittedIdleTime) {
            getSender().close();
        }
    }


    /**
     *
     */
    private void sendAYTIfIdleForTooLong() {
      if (getSilentTimeForIncoming() > idleTimeBeforeAYT) {
         try {
            getSender().send(ARE_YOU_THERE);
         } catch (IOException ioe) {
            getSender().close();
         }
      }
   }
}