package keepalive.keepalive;

import java.io.IOException;

import keepalive.communication.MessageListener;
import keepalive.communication.Receiver;
import keepalive.communication.Sender;

/**
 * This "Keep-Alive" does not actually do anything for cleaning up the local socket;
 * rather, it helps the remote Are-You-There protocol by waiting for incoming
 * Are-You-Theres and responding to them.
 *
 * @author Joshua Fox
 */
public class IHearYou extends KeepAlive implements MessageListener {

    /**
     *
     */
    public IHearYou(Receiver receiver, Sender sender) {
        super(receiver, sender);
    }

    /**
     *
     */
    public IHearYou() {
    }


    /**
     * Set the receiver object which receives messages that arrive
     * at the socket associated with this Keep-Alive.
     * @param receiver
     */
    public void setReceiver(Receiver receiver) {
        super.setReceiver(receiver);
        receiver.setMessageListener(this);
    }

    /**
     *
     */
    public void run() {
        try {
            sleep(Long.MAX_VALUE);
        } catch (InterruptedException ie) {
        }

    }


    /**
     * This method is called by Receiver as the call-back method
     * from MessageListener
     *
     * Ignore all messages but "Are You There." Respond to Are-You-There
     * with "I Hear You."
     *
     * "I Hear you" is expressed here by a dummy message,
     * since its only purpose is to update the last-message-received
     * timestamp for the other side.
     *
     * Our messages are sent as Strings. In real applications you will
     * want a more sophisticated encapsulation of messages, perhaps as
     * serialized objects.
     */
    public boolean messageReceived(String s) {
        if (s.equals(ARE_YOU_THERE)) {
            try {
                getSender().send(I_HEAR_YOU);
            } catch (IOException ioe) {
                getSender().close();
            }
            return true;
        }
        return false;
    }
}
