package keepalive.keepalive;

import java.io.IOException;

import keepalive.communication.MessageListener;
import keepalive.communication.Receiver;
import keepalive.communication.Sender;

/**
 * Combines PassiveKeepAlive with I-Hear-You responses to incoming Are-You-Theres.
 * Used with ActiveKeepAliveWithAYT. Serves as an example of combining Keep-Alives
 * in a single application, as will often happen in real programs.
 *
 * @see keepalive.keepalive.PassiveKeepAlive
 * @see keepalive.keepalive.IHearYou
 * @see ActiveKeepAliveWithAYT
 *
 * @author Joshua Fox
 */
public class PassiveKeepAliveWithIHY extends KeepAlive implements MessageListener{
    /**
     *
     */
    public PassiveKeepAliveWithIHY(Receiver receiver, Sender sender) {
        super(receiver, sender);
    }


    /**
     *
     */
    public PassiveKeepAliveWithIHY() {
    }


    /**
     *
     */
    public void setReceiver(Receiver receiver) {
        super.setReceiver(receiver);
        receiver.setMessageListener(this);
    }

    /**
     *
     */
    public void run() {
       while (!endRequested()) {

          try {
                // Only try the connection if you have been waiting for a long time.
                // This will not happen if the other side's probes are
                // coming regularly. If this dummy message is successfully sent,
                // the other side's Are You There will never be sent.
                if (System.currentTimeMillis() - getReceiver().lastTimeStamp() >
                      2 * TIME_BETWEEN_ACTIVE_PROBES) {
                            getSender().send(getDummyString());
                }
            } catch (IOException ioe) {

                getSender().close();
            }

            try {
                // Every now and then, wake up to check
                // the situation.
                this.sleep((long)(0.5 * TIME_BETWEEN_ACTIVE_PROBES));
            } catch (InterruptedException ie) { }
       }
    }


    /**
     * This method examines all incoming messages, checking for
     * Are-You-Theres. This method is called Receiver as the call-back method
     * from MessageListener
     *
     * Ignore all messages but "Are You There." Respond to Are-You-There
     * with I-Hear-You.
     * "I Hear You" could have been expressed here by a dummy message,
     * since its only purpose is to update the last-message-received
     * timestamp for the Are-You-There side.
     */
    public boolean messageReceived(String s) {
        if (s.equals(ARE_YOU_THERE)) {
            try {
                getSender().send(I_HEAR_YOU);
            } catch (IOException ioe) {
                getSender().close();
            }
           
            return true;
        }
       
        return false;
    }
}
