package keepalive.socket.adapter;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import keepalive.socket.ServerSocketIfc;
import keepalive.socket.AbstractSocket;

/**
 * An adapter class that wraps a TCP/IP ServerSocket to allow it to be used
 * with our ServerSocket interface.
 *
 * @author Joshua Fox
 */
public class ServerSocketAdapter implements ServerSocketIfc {
    /**
     * The ServerSocket which this adapter wraps.
     *
     */
    private ServerSocket serverSocket;

    /**
     * Create new ServerSocket adapter for a ServerSocket that will be 
     * bound to the given port.
     *
     * @exception IOException if an I/O error occurs.
     * @param port
     */
    public ServerSocketAdapter(int port) throws IOException {
        serverSocket = new ServerSocket(port);
    }

   /**
    * Accept a connection and spin off a new socket.
    *
    * @exception IOException
    * @return the spun-off socket.
    */
   public AbstractSocket accept() throws IOException {
        Socket socket = serverSocket.accept();
        return new SocketAdapter(socket);

   }

   /**
    * Close this ServerSocket.
    *
    * @exception IOException
    */
   public void close() throws IOException{
        serverSocket.close();
   }

    /**      *
     */
    public String toString() {
        String s = this.getClass().getName();
        if (serverSocket != null) {
            s += "-" + serverSocket;
        }
        return s;
    }
}
