package keepalive.socket.sim;

/**
 * This interface represents a physical layer that can be unplugged.
 * The metaphor here is to the Ethernet plug on your computer, although
 * the network can be partitioned in other less trivial ways.
 */
interface Pluggable {
    /**
     * Determine whether this object is plugged in.
     *
     * @return true iff this object is plugged in.
     */
    boolean isPluggedIn() ;
    /**
     * Plug in or unplug this object.
     *
     * @param b true to plug in this object, or false to unplug it.
     */
    void setPluggedIn(boolean b);
}
