package keepalive.ui;

import java.awt.*;
import java.awt.event.*;

/**
 * The Frame which holds within it the Keep-Alive user interface.
 * The Frame itself has no UI features other than the KeepAliveUI.
 * @see keepalive.ui.KeepAliveUI
 */
public class KeepAliveFrame extends Frame {

       /**
        * Construct and show a new KeepAliveFrame
        * @param ui the Keep-Alive user interface that this frame will hold.
        * @param serverSide true if this Frame is on the server-side, false if 
        * it is on the client-side.
        */
       public KeepAliveFrame(KeepAliveUI ui, boolean serverSide) {
            setTitle("KeepAlive: " + (serverSide?"Server":"Client"));
            addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent ev) {
                    dispose();
                    System.exit(0);
                }
            });

            add(ui, BorderLayout.CENTER);
            Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

            int horizSize = (int)(0.5 * screenSize.width);
            int vertSize =  (int)(0.8 * screenSize.height);
            setSize(horizSize, vertSize);

            double horizFraction = serverSide ? 0.0 : 0.5;
            int horizLoc = (int)(horizFraction * screenSize.width);
            int vertLoc = (int)(0.1 * screenSize.height);
            setLocation(horizLoc, vertLoc);

            setVisible(true);
       }
}
