package keepalive.ui;

import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import keepalive.communication.Output;

/**
 * User interface for displaying incoming and outgoing messages
 * and the name of the  Keep-Alive class that is now in use.
 */
public abstract class KeepAliveUI extends Panel implements MessageDisplayer{
    private Output output;
    private Button sendButton = new Button();
    private TextArea outputConsole = new TextArea();
    private TextArea inputConsole = new TextArea();
    private Label keepAliveLabel = new Label();

    /**
     * This field is "protected" so that subclasses can add more buttons.
     */
    protected Panel buttonPanel = new Panel();

   
    /**
     *
     */
    public KeepAliveUI() {
        this.setLayout(new BorderLayout());

        // buttonPanel
        this.add(buttonPanel, BorderLayout.NORTH);

        // sendButton

        sendButton.setLabel("Send A Message");
        sendButton.addActionListener(new ActionListener() {
            /**              */
            public void actionPerformed(ActionEvent ev) {
                sendButton_actionPerformed(ev);
            }
        });
        sendButton.setEnabled(false);
        buttonPanel.add(sendButton, null);


        // consolePanel
        Panel consolePanel = new Panel();
        consolePanel.setLayout(new BorderLayout());
        this.add(consolePanel, BorderLayout.CENTER);


        //consoleLabelPanel
        Panel consoleLabelPanel = new Panel();
        consoleLabelPanel.setLayout(new GridLayout(1,2));
        Label sendLabel = new Label("Sent");
        sendLabel.setBackground(Color.lightGray);
        sendLabel.setAlignment(Label.CENTER);
        Label receiveLabel = new Label("Received");
        receiveLabel.setBackground(Color.lightGray);
        receiveLabel.setAlignment(Label.CENTER);
        consoleLabelPanel.add(sendLabel);
        consoleLabelPanel.add(receiveLabel);
        consolePanel.add(consoleLabelPanel, BorderLayout.NORTH);
        consolePanel.add( keepAliveLabel, BorderLayout.SOUTH);

        // innerConsolePanel
        Panel innerConsolePanel = new Panel();
        innerConsolePanel.setLayout(new GridLayout(1,2));
        innerConsolePanel.add(outputConsole, null);
        innerConsolePanel.add(inputConsole, null);
        consolePanel.add(innerConsolePanel, BorderLayout.CENTER);

    }

    /**
     *
     */
    public void setOutputObject(Output output) {
        sendButton.setEnabled(true);
        output.setMessageDisplayer(this);
        this.output = output;

    }


    /**
     *
     */
    void sendButton_actionPerformed(ActionEvent ev) {
        sendButton.setEnabled(false);
        try {
            output.sendMessage();
        } catch (IOException ioe) {
            output.close();
        }

        sendButton.setEnabled(true);
    }

    /**
     *
     */
    public void setKeepAliveDescription(String keepAliveDescription) {
        keepAliveLabel.setText( keepAliveDescription);
    }


    /**
     * From MessageUI
     */
    public void display(String participantName, 
                                            String message, boolean sending) {
        if (sending) {
            outputConsole.append(message + "\n");
        } else {
            inputConsole.append(message + "\n");
        }
    }
}
