// htmlreader.cpp
// copyright (C) 2000 by David Cox All Rights Reserved

#include <windows.h>
#include <memory.h>
#include <string.h>
#include <io.h>
#include <iostream.h>
#include "htmlreader.h"
#include "cereserror.h"


HtmlReader::HtmlReader()
{
	m_buffer = new char[e_bufferSize];
	if (m_buffer == 0)
	{
		CeresError e;
		e.Msg("Cannot allocate memory for HtmlReader");
		e.Line(__LINE__);
		e.File(__FILE__);
		throw (e);
	}
}


HtmlReader::HtmlReader(const HtmlReader & r)
{
	// allocate buffer for file
	m_buffer = new char[e_bufferSize];
	if (m_buffer == 0)
	{
		CeresError e;
		e.Msg("Cannot allocate memory for HtmlReader");
		e.Line(__LINE__);
		e.File(__FILE__);
		throw (e);
	}

	// copy reader
	memcpy(m_buffer, r.m_buffer, e_bufferSize);
}


HtmlReader::~HtmlReader()
{
	delete [] m_buffer;
}


HtmlReader &HtmlReader::operator=(const HtmlReader & r)
{
	if (this == &r) return *this;

	// get rid of current buffer
	if (m_buffer) delete [] m_buffer;

	// allocate buffer for file
	m_buffer = new char[e_bufferSize];
	if (m_buffer == 0)
	{
		CeresError e;
		e.Msg("Cannot allocate memory for HtmlReader");
		e.Line(__LINE__);
		e.File(__FILE__);
		throw (e);
	}

	// copy reader
	memcpy(m_buffer, r.m_buffer, e_bufferSize);

	return *this;
}


void HtmlReader::Read(char * filename)
{
	// try to open the file
	FILE * file = fopen(filename, "r");

	// if file won't open, throw exception
	if (!file)
	{
		CeresError e;
		e.Line(__LINE__);
		e.File(__FILE__);
		throw (e);
	}

	// ok, file is open
	// clear the buffer and read the file's contents into the buffer
	// terminate with a 0
	memset(m_buffer, 0, e_bufferSize);
	int fh = _fileno(file);
	int s = _read(fh, m_buffer, e_bufferSize-1);
	m_buffer[s] = 0;

	// close the file
	fclose(file);
}


int HtmlReader::Search(char * target)
{
	int len = 0;
	int pos = 0;

	// search for a string in the buffer
	char * t = strstr(m_buffer, target);
	if (t) pos = t - m_buffer;

	return pos;
}


// return a character from the buffer
char HtmlReader::operator [] (int n)
{
	return (m_buffer[n]);
}


void HtmlReader::Translate(CommandTable & command)
{
	char * ptr = m_buffer;
	while (*ptr != 0)
	{
		// look ahead for start of command
		if (strnicmp("<!-- !", ptr, 6) == 0)
		{
			// parse the command
			ptr = m_parser.Parse(ptr);

			// Do the command by getting it and its parameters
			// from the parser and passing them to the command
			// object. If the command is known by the command
			// object, the object will act on it. Otherwise the
			// object does nothing.
			command.Do(m_parser.Command(), m_parser.Param());
		}
		else
		{
			cout << *ptr;
			ptr++;
		}
	}
}