// pair.cpp
// copyright (C) 2000 by David Cox All Rights Reserved

#include "pair.h"
#include "cereserror.h"
#include <iostream.h>
#include <string.h>


PairList::PairList()
{
	m_top = m_bot = 0;
	m_size = 0;
}


PairList::PairList(const PairList & l)
{
	Pair * tmp = l.m_top;
	while (tmp != 0)
	{
		// create a new pair
		Pair * newPair = new Pair;

		// if allocation fails,
		// destroy pair list and throw exception
		if (!newPair)
		{
			Destroy();
			CeresError e;
			e.Msg("Cannot allocate memory for pair list");
			e.Line(__LINE__);
			e.File(__FILE__);
			throw (e);
		}

		// copy tmp into new pair
		*newPair = *tmp;

		// add pair to list
		Add(newPair);

		// repeat for next pair
		tmp = tmp->m_next;
	}
}


PairList::~PairList()
{
	Destroy();
}


PairList & PairList::operator=(const PairList & l)
{
	if (this == &l) return *this;

	// get rid of existing list
	Destroy();

	// copy list
	Pair * tmp = l.m_top;
	while (tmp != 0)
	{
		// create a new pair
		Pair * newPair = new Pair;

		// if allocation fails,
		// destroy pair list and throw exception
		if (!newPair)
		{
			Destroy();
			CeresError e;
			e.Msg("Cannot allocate memory for pair list");
			e.Line(__LINE__);
			e.File(__FILE__);
			throw (e);
		}

		// copy tmp into new pair
		*newPair = *tmp;

		// add pair to list
		Add(newPair);

		// repeat for next pair
		tmp = tmp->m_next;
	}

	return *this;
}


void PairList::Add(Pair * pair)
{
	pair->m_next = 0;

	if (m_bot)
	{
		m_bot->m_next = pair;
		m_bot = pair;
	}
	else
	{
		m_top = m_bot = pair;
	}

	m_size++;
}


Pair * PairList::operator[](int i)
{
	if ((i >= 0) && (i < m_size))
	{
		Pair * cur = m_top;
		int n = 0;
		while (n != i)
		{
			cur = cur->m_next;
			n++;
		}
		return cur;
	}

	return 0;
}


void PairList::Destroy()
{
	while (m_top != 0)
	{
		Pair * cur = m_top;
		m_top = m_top->m_next;
		delete cur;
	}
	m_size = 0;
	m_bot = 0;
}


int PairList::Size()
{
	return m_size;
}


void PairList::PrintList()
{
	for (int i = 0; i < Size(); i++)
	{
		Pair * p = (*this)[i];
		if (p->m_name)
			cout << p->m_name << ":";
		if (p->m_value)
			cout << p->m_value << endl;
	}
}



void PairList::CreateList(char * input, char * tokenstr)
{
	Destroy();

	char * pToken;

	// create pair list
	pToken = strtok(input, tokenstr);
	while (pToken != NULL)
	{
		Pair * p = new Pair;
		if (!p)
		{
			CeresError e;
			e.Msg("Cannot allocate memory for pair list");
			e.Line(__LINE__);
			e.File(__FILE__);
			throw (e);
		}

		p->m_value = 0;
		p->m_name = pToken;
		Add(p);

		pToken = strtok(NULL, tokenstr);
	}
}


// extract values from m_env.m_query and place
// pointers to values into m_queryList
void PairList::SetValues(char * tokenstr)
{
	for (int i = 0; i < Size(); i++)
	{
		Pair * p = (*this)[i];
		char * pToken = strtok(p->m_name, tokenstr);
		if (pToken != NULL)
		{
			pToken = strtok(NULL, tokenstr);
			p->m_value = pToken;
		}
	}
}
