// sqltojavascriptparser.cpp
// copyright (C) 2000 by David Cox All Rights Reserved

#include <windows.h>
#include <memory.h>
#include <string.h>
#include <iostream.h>
#include <ctype.h>
#include "cgi.h"
#include "sqltojavascriptparser.h"
#include "cereserror.h"

extern Cgi cgi;

SqlToJavascriptParser::SqlToJavascriptParser()
{
	memset(m_queryName, 0, e_queryNameSize);
	memset(m_sql, 0, e_sqlSize);
	memset(m_datasource, 0, e_datasourceSize);
	memset(m_userid, 0, e_useridSize);
	memset(m_password, 0, e_passwordSize);
	m_page = 0;
	m_pagesize = 0;
	m_nRows = 0;
	m_table = 0;
}


SqlToJavascriptParser::SqlToJavascriptParser(const SqlToJavascriptParser & p)
{
	memcpy(m_queryName, p.m_queryName, e_queryNameSize);
	memcpy(m_sql, p.m_sql, e_sqlSize);
	memcpy(m_datasource, p.m_datasource, e_datasourceSize);
	memcpy(m_userid, p.m_userid, e_useridSize);
	memcpy(m_password, p.m_password, e_passwordSize);
	m_page = p.m_page;
	m_pagesize = p.m_pagesize;
	m_nRows = p.m_nRows;
	m_table = new Table(*p.m_table);
}


SqlToJavascriptParser::~SqlToJavascriptParser()
{
	delete m_table;
}


SqlToJavascriptParser &SqlToJavascriptParser::operator=(const SqlToJavascriptParser & p)
{
	if (this == &p) return *this;

	if (m_table) delete m_table;
	memcpy(m_queryName, p.m_queryName, e_queryNameSize);
	memcpy(m_sql, p.m_sql, e_sqlSize);
	memcpy(m_datasource, p.m_datasource, e_datasourceSize);
	memcpy(m_userid, p.m_userid, e_useridSize);
	memcpy(m_password, p.m_password, e_passwordSize);
	m_page = p.m_page;
	m_pagesize = p.m_pagesize;
	m_nRows = p.m_nRows;
	m_table = new Table(*p.m_table);

	return *this;
}


void SqlToJavascriptParser::Substitute(const char * param, int & p, int & s)
{
	if (!param) return;

	// get variable name
	char var[e_substituteSize];
	memset(var, 0, e_substituteSize);

	p++;
	int i = 0;
	while (param[p] && !isspace(param[p]) && (i < e_substituteSize))
	{
		var[i] = param[p];
		p++;
		i++;
	}

	// get value of variable
	char * val = cgi.GetQueryValue(var);
	if (!val) return;

	// append value to m_sql
	i = 0;
	while (val[i] && !isspace(val[i]) && (s < e_sqlSize))
	{
		m_sql[s] = val[i];
		i++;
		s++;
	}
}


void SqlToJavascriptParser::SaveQueryName(const char * param, int & p)
{
	int start = p; // save the starting point

	// skip white space
	while (isspace(param[p])) p++;

	// save query name
	int s = 0;
	memset(m_queryName, 0, e_queryNameSize);
	while (!isspace(param[p]) && (s < e_queryNameSize-1))
	{
		m_queryName[s] = param[p];
		p++;
		s++;
	}

	if (stricmp(m_queryName, "select") == 0)
	{
		// oops, we started the select statement
		// revert back to start
		p = start;
	}
}


void SqlToJavascriptParser::SaveDatasource(const char * param, int & p)
{
	int start = p; // save the starting point

	// skip white space
	while (isspace(param[p])) p++;

	// save datasource name
	int s = 0;
	memset(m_datasource, 0, e_datasourceSize);
	while (!isspace(param[p]) && (s < e_datasourceSize-1))
	{
		m_datasource[s] = param[p];
		p++;
		s++;
	}

	if (stricmp(m_datasource, "select") == 0)
	{
		// oops, we started the select statement
		// revert back to start
		p = start;
		memset(m_datasource, 0, e_datasourceSize);
	}
}


void SqlToJavascriptParser::SaveUserid(const char * param, int & p)
{
	int start = p; // save the starting point

	// skip white space
	while (isspace(param[p])) p++;

	// save query name
	int s = 0;
	memset(m_userid, 0, e_useridSize);
	while (!isspace(param[p]) && (s < e_useridSize-1))
	{
		m_userid[s] = param[p];
		p++;
		s++;
	}

	if (stricmp(m_userid, "select") == 0)
	{
		// oops, we started the select statement
		// revert back to start
		p = start;
		memset(m_userid, 0, e_useridSize);
	}
}


void SqlToJavascriptParser::SavePassword(const char * param, int & p)
{
	int start = p; // save the starting point

	// skip white space
	while (isspace(param[p])) p++;

	// save query name
	int s = 0;
	memset(m_password, 0, e_passwordSize);
	while (!isspace(param[p]) && (s < e_passwordSize-1))
	{
		m_password[s] = param[p];
		p++;
		s++;
	}

	if (stricmp(m_password, "select") == 0)
	{
		// oops, we started the select statement
		// revert back to start
		p = start;
		memset(m_password, 0, e_passwordSize);
	}
}


void SqlToJavascriptParser::SaveQuery(const char * param, int & p)
{
	// skip white space
	while (isspace(param[p])) p++;

	// save the query
	// if an @ is encountered, performed substitution
	int s = 0;
	memset(m_sql, 0, e_sqlSize);
	while ((param[p] != 0) && (p < e_sqlSize))
	{
		if (param[p] == '@')
		{
			Substitute(param, p, s);
		}
		else
		{
			m_sql[s] = param[p];
		}

		s++;
		p++;
	}
}


void SqlToJavascriptParser::SavePage()
{
	// get page number
	char * s = cgi.GetQueryValue("page");
	if (s == 0)
	{
		// default is page 0
		m_page = 0;
	}
	else
	{
		m_page = atoi(s);
	}
}


void SqlToJavascriptParser::SavePageSize()
{
	// get page number
	char * s = cgi.GetQueryValue("pagesize");
	if (s == 0)
	{
		// default size is 10
		m_pagesize = 10;
	}
	else
	{
		m_pagesize = atoi(s);

		// set pagesize to 10 if something goofy happens
		if (m_pagesize < 1) m_pagesize = 10;
	}
}


void SqlToJavascriptParser::ParseParam(const char * param)
{
	// see if query name is available
	if (param == 0) throw MissingQueryName();

	int p = 0;
	SaveQueryName(param, p);
	SaveDatasource(param, p);
	SaveUserid(param, p);
	SavePassword(param, p);
	SaveQuery(param, p);
	SavePage();
	SavePageSize();
}


void SqlToJavascriptParser::Select()
{
	if (m_table != 0) delete m_table;

	// get result table
	m_table = new Table(m_pagesize);
	m_database.Datasource(m_datasource);
	m_database.OpenDatabase();
	m_database.Select(m_sql, m_page, *m_table);
	m_database.CloseDatabase();
}


void SqlToJavascriptParser::PrintResultTable(const char * param)
{
	try
	{
		// parse param to separate sql from page number
		ParseParam(param);

		// execute the sql to get the data
		Select();

		// output the result table
		cout << endl; // start on a new line
		cout << "var " << m_queryName << " = new Object();" << endl;

		for (int col = 0; col < m_table->NumCols(); col++)
		{
			// declare javascript array for col
			Column &c = m_table->GetCol(col);
			cout << m_queryName << "." << c.Name() << " = ";
			cout << "Array();" << endl;

			for (int row = 0; row < m_table->NumRows(); row++)
			{
				cout << m_queryName << "." << c.Name();
				cout << "[" << row << "]" << " = ";
				cout << "\"" << c.Data(row) << "\"" << ";" << endl;
			}

			cout << endl;
		}
	}

	catch(MissingQueryName &)
	{
		// generate a javascript comment
		cout << "// Missing Query Name" << endl;
	}

	catch(MissingQuery &)
	{
		// generate a javascript comment
		cout << "// Missing Query" << endl;
	}

}

