//
//=======================================================================
// Copyright 1997, 1998, 1999, 2000 University of Notre Dame.
// Authors: Andrew Lumsdaine, Lie-Quan Lee, Jeremy G. Siek
//
// This file is part of the Generic Graph Component Library
//
// You should have received a copy of the License Agreement for the
// Generic Graph Component Library along with the software;  see the
// file LICENSE.  If not, contact Office of Research, University of Notre
// Dame, Notre Dame, IN  46556.
//
// Permission to modify the code and to distribute modified code is
// granted, provided the text of this NOTICE is retained, a notice that
// the code was modified is included with the above COPYRIGHT NOTICE and
// with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
// file is distributed with the modified code.
//
// LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
// By way of example, but not limitation, Licensor MAKES NO
// REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
// PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
// OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
// OR OTHER RIGHTS.
//=======================================================================
//

#ifndef GGCL_BELLMAN_FORD_SHORTEST_PATHS_HPP
#define GGCL_BELLMAN_FORD_SHORTEST_PATHS_HPP

#include <boost/graph_structure.hpp>
#include <ggcl/ggcl_config.h>
#include <ggcl/relax.h>
#include <ggcl/ggcl_concepts.h>
#include <ggcl/properties.h>

namespace ggcl {

  // Variant (2)
  template <class EdgeListGraph, class Size, 
            class Weight, class Distance, class Visitor>
  bool bellman_ford_shortest_paths(EdgeListGraph& g, Size N, Weight w, 
                                   Distance d, Visitor visit)
  {
    REQUIRE(EdgeListGraph, EdgeListGraph);
    typedef typename boost::graph_traits<EdgeListGraph>::edge_descriptor Edge;
    typedef typename boost::graph_traits<EdgeListGraph>::vertex_descriptor
      Vertex;
    REQUIRE2(Distance, Vertex, ReadWritePropertyAccessor);
    REQUIRE2(Weight, Edge, ReadablePropertyAccessor);
    typedef typename boost::property_traits<Distance>::value_type D_value;
    typedef typename boost::property_traits<Weight>::value_type W_value;
    REQUIRE(D_value, LessThanComparable);
    REQUIRE3(D_value, D_value, W_value, _OP_PLUS);

    typename boost::graph_traits<EdgeListGraph>::edge_iterator i, end;
    
    for (Size k = 0; k < N; ++k)
      for (boost::tie(i,end) = edges(g); i != end; ++i)
        if (relax(*i, g, w, d))
          visit.process(*i,g);

    for (boost::tie(i,end) = edges(g); i != end; ++i)
      if ( d[source(*i,g)] + w[*i] < d[target(*i,g)] )
        return false;

    return true;
  }

  // Variant (1)
  template <class EdgeListGraph, class Size, 
            class Weight, class Distance>
  bool bellman_ford_shortest_paths(EdgeListGraph& g, Size N, 
				   Weight w, Distance d)
  {
    return bellman_ford_shortest_paths(g, N, w, d, null_visitor());
  }

} /* namespace ggcl */

#endif /* GGCL_BELLMAN_FORD_SHORTEST_PATHS_HPP */
