//
//=======================================================================
// Copyright 1997, 1998, 1999, 2000 University of Notre Dame.
// Authors: Andrew Lumsdaine, Lie-Quan Lee, Jeremy G. Siek
//
// This file is part of the Generic Graph Component Library
//
// You should have received a copy of the License Agreement for the
// Generic Graph Component Library along with the software;  see the
// file LICENSE.  If not, contact Office of Research, University of Notre
// Dame, Notre Dame, IN  46556.
//
// Permission to modify the code and to distribute modified code is
// granted, provided the text of this NOTICE is retained, a notice that
// the code was modified is included with the above COPYRIGHT NOTICE and
// with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
// file is distributed with the modified code.
//
// LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
// By way of example, but not limitation, Licensor MAKES NO
// REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
// PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
// OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
// OR OTHER RIGHTS.
//=======================================================================
//
#ifndef GGCL_VERTEX_VISITOR_H
#define GGCL_VERTEX_VISITOR_H

#include <ggcl/utils.h>
#include <ggcl/properties.h>
#include <ggcl/foundation_visitor_aux.h>
#include <ggcl/directed_category_traits.h>

namespace ggcl {

  template < class Color, class Base >
  class bfs_visitor {
    typedef bfs_visitor self;
  public:
    inline bfs_visitor(Color c, const Base& b) 
      : base(b), color(c) {}

    template <class Vertex>
    inline void initialize(Vertex u) {
      color[u] = white( color[u] );
      visitor_initialize_evaluate( base, u );
    }

    template <class Vertex>
    inline void start(Vertex u) {
      color[u] = gray( color[u] );
      visitor_start_evaluate( base, u );
    }
  
    template <class Vertex>
    inline void discover(Vertex u) {
      visitor_discover_evaluate( base, u );
    }

    template <class Vertex>
    inline void finish(Vertex u) {
      color[u] = black( color[u] );
      visitor_finish_evaluate( base, u ); 
    }

    template <class Edge, class Graph>
    inline bool process(Edge e, Graph& g) {
      typedef typename boost::graph_traits<Graph>::vertex_descriptor Vertex;
      Vertex v = target(e, g);
      bool ret = is_undiscovered(v);
      if ( ret ) {
	color[v] = gray( color[v] ); 
	visitor_process_evaluate(base, e, g, tree_edge_tag());
      } else {
	typedef typename boost::graph_traits<Graph>::directed_category Cat;
	process_evaluate(e, g, Cat());
      }
      visitor_process_evaluate(base, e, g, every_edge_tag());
      return ret;
    }

    template <class Vertex>
    inline bool is_undiscovered(Vertex u) {
      return (color[u] == white( color[u] ));
    }

  private:
    template <class Edge, class Graph>
    inline void process_evaluate(Edge e, Graph& g, directed_tag) {
      visitor_process_evaluate(base, e, g, back_or_cross_edge_tag());
    }

    template <class Edge, class Graph>
    inline void process_evaluate(Edge e, Graph& g, undirected_tag) {
      visitor_process_evaluate(base, e, g, cross_edge_tag());
    }

  protected:
    Base base;
    Color color;
  };

  template <class Color, class UserVisitorList>
  bfs_visitor<typename detail::property_accessor_adaptor<Color>::type, 
              UserVisitorList>
  visit_bfs(Color c, UserVisitorList b) {
    typedef typename detail::property_accessor_adaptor<Color>::type ColorPA;
    return bfs_visitor<ColorPA, UserVisitorList>(c, b);
  }

#if !defined GGCL_MSVCPP
  template <class Color, class UserVisitorList>
  bfs_visitor<random_access_iterator_property_accessor<Color*,Color,Color&>, 
              UserVisitorList>
  visit_bfs(Color* c, UserVisitorList b) {
    typedef random_access_iterator_property_accessor<
      Color*,Color,Color&> ColorPA;
    return bfs_visitor<ColorPA, UserVisitorList>(c, b);
  }
#endif

  template <class Color, class UserVisitorList>
  bfs_visitor<random_access_iterator_property_accessor<Color*,Color,Color&>, 
              UserVisitorList>
  visit_bfs_ptr(Color* c, UserVisitorList b) {
    typedef random_access_iterator_property_accessor<
      Color*,Color,Color&> ColorPA;
    return bfs_visitor<ColorPA, UserVisitorList>(c, b);
  }

}


#endif /*GGCL_VERTEX_VISITOR_H*/
