//
//=======================================================================
// Copyright 1997, 1998, 1999, 2000 University of Notre Dame.
// Authors: Andrew Lumsdaine, Lie-Quan Lee, Jeremy G. Siek
//
// This file is part of the Generic Graph Component Library
//
// You should have received a copy of the License Agreement for the
// Generic Graph Component Library along with the software;  see the
// file LICENSE.  If not, contact Office of Research, University of Notre
// Dame, Notre Dame, IN  46556.
//
// Permission to modify the code and to distribute modified code is
// granted, provided the text of this NOTICE is retained, a notice that
// the code was modified is included with the above COPYRIGHT NOTICE and
// with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
// file is distributed with the modified code.
//
// LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
// By way of example, but not limitation, Licensor MAKES NO
// REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
// PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
// OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
// OR OTHER RIGHTS.
//=======================================================================
//
#ifndef STL_CONCEPT_CHECKS3_H
#define STL_CONCEPT_CHECKS3_H

#include <iterator>
#include <ggcl/ggcl_config.h>

#if defined(__GNUC__) || defined(__KCC)
#define _FPTR &
#else
#define _FPTR
#endif

#if defined GGCL_MSVCPP || defined __ICL

#define REQUIRE(__type_var, __concept)

#define REQUIRE2(__type_var1, __type_var2, __concept)

#define REQUIRE3(__type_var1, __type_var2, __type_var3,__concept)

#define REQUIRE4(__type_var1, __type_var2, __type_var3,__type_var4,__concept)

#define CLASS_REQUIRES(__tv, __concept)

#define CLASS_REQUIRES2(__tv1, __tv2, __concept)

#define CLASS_REQUIRES3(__tv1, __tv2, __tv3, __concept)

#define CLASS_REQUIRES4(__tv1, __tv2, __tv3, __tv4, __concept)

#define CT_ASSERT( B )

#else
#define REQUIRE(__type_var, __concept) \
do { \
  void (__concept##_concept <__type_var>::*__x)() = \
    _FPTR __concept##_concept <__type_var>::constraints; \
  __x = __x; } while (0)

#define REQUIRE2(__type_var1, __type_var2, __concept) \
do { \
  void (__concept##_concept <__type_var1, __type_var2>::*__x)() = \
    _FPTR __concept##_concept <__type_var1, __type_var2>::constraints; \
  __x = __x; } while (0)

#define REQUIRE3(__type_var1, __type_var2, __type_var3,__concept) \
do { \
  void (__concept##_concept <__type_var1, __type_var2, __type_var3>::*__x)() = \
    _FPTR __concept##_concept <__type_var1, __type_var2, __type_var3>::constraints; \
  __x = __x; } while (0)

#define REQUIRE4(__type_var1, __type_var2, __type_var3,__type_var4,__concept) \
do { \
 void (__concept##_concept <__type_var1,__type_var2,__type_var3,__type_var4>::*__x)() = \
 _FPTR __concept##_concept <__type_var1, __type_var2, __type_var3, __type_var4>::constraints;\
  __x = __x; } while (0)

#define CLASS_REQUIRES(__tv, __concept) \
  typedef void (__concept##_concept <__tv>::* __func##__tv##__concept)(); \
  template <__func##__tv##__concept _Tp1> \
  struct __dummy_struct_##__tv##__concept { }; \
  static __dummy_struct_##__tv##__concept< \
    _FPTR __concept##_concept <__tv>::constraints>  \
  __dummy_ptr_##__tv##__concept

#define CLASS_REQUIRES2(__tv1, __tv2, __concept) \
  typedef void (__concept##_concept <__tv1,__tv2>::* __func##__tv1##__tv2##__concept)(); \
  template <__func##__tv1##__tv2##__concept _Tp1> \
  struct __dummy_struct_##__tv1##__tv2##__concept { }; \
  static __dummy_struct_##__tv1##__tv2##__concept< \
    _FPTR __concept##_concept <__tv1,__tv2>::constraints>  \
  __dummy_ptr_##__tv1##__tv2##__concept

#define CLASS_REQUIRES3(__tv1, __tv2, __tv3, __concept) \
  typedef void (__concept##_concept <__tv1,__tv2,__tv3>::* __func##__tv1##__tv2##__tv3##__concept)(); \
  template <__func##__tv1##__tv2##__tv3##__concept _Tp1> \
  struct __dummy_struct_##__tv1##__tv2##__tv3##__concept { }; \
  static __dummy_struct_##__tv1##__tv2##__tv3##__concept< \
    _FPTR __concept##_concept <__tv1,__tv2,__tv3>::constraints>  \
  __dummy_ptr_##__tv1##__tv2##__tv3##__concept

#define CLASS_REQUIRES4(__tv1, __tv2, __tv3, __tv4, __concept) \
  typedef void (__concept##_concept <__tv1,__tv2,__tv3,__tv4>::* __func##__tv1##__tv2##__tv3##__tv4##__concept)(); \
  template <__func##__tv1##__tv2##__tv3##__tv4##__concept _Tp1> \
  struct __dummy_struct_##__tv1##__tv2##__tv3##__tv4##__concept { }; \
  static __dummy_struct_##__tv1##__tv2##__tv3##__tv4##__concept< \
    _FPTR __concept##_concept <__tv1,__tv2,__tv3,__tv4>::constraints>  \
  __dummy_ptr_##__tv1##__tv2##__tv3##__tv4##__concept

// compile-time assertion
template <bool> struct ct_assert;
template <> struct ct_assert<true> { };
template <int> struct ct_assert_test { };

#define CT_ASSERT( B )\
 typedef ct_assert_test<sizeof(ct_assert< ( B ) >)> \
 _boost_postulate_details_##__LINE__

//#define REQUIRE_SAME_TYPE(X,Y) CT_ASSERT(typeid(X) == typeid(Y))
// this doesn't work?


template <class T> void ignore_unused_variable_warning(const T&) { }

  template <class T>
  struct Integer_concept {
    void constraints() { 
#ifdef GGCL_PARTIAL_SPECIALIZATION
      error_the_type_must_be_an_integer_type();
#endif      
    }
  };
#ifdef GGCL_PARTIAL_SPECIALIZATION
  template <> struct Integer_concept<short> { void constraints() {} };
  template <> struct Integer_concept<unsigned short> { void constraints() {} };
  template <> struct Integer_concept<int> { void constraints() {} };
  template <> struct Integer_concept<unsigned int> { void constraints() {} };
  template <> struct Integer_concept<long> { void constraints() {} };
  template <> struct Integer_concept<unsigned long> { void constraints() {} };
  // etc.
#endif      

  template <class X, class Y>
  struct Convertible_concept
  {
    void constraints() {
      Y y = x;
      ignore_unused_variable_warning(y);
    }
    X x;
  };

  template <class TT>
  struct Assignable_concept
  {
    void constraints() {
      a = a;              // require assignment operator
      TT c(a);            // require copy constructor
      const_constraints(a);
      ignore_unused_variable_warning(c);
    }
    void const_constraints(const TT& b) {
      a = b;              // const required for argument to assignment
      TT c(b);            // const required for argument to copy constructor
      ignore_unused_variable_warning(c);
    }
    TT a;
  };

  template <class TT>
  struct DefaultConstructible_concept
  {
    void constraints() {
      TT a;               // require default constructor
      ignore_unused_variable_warning(a);
    }
  };

  template <class T>
  struct CopyConstructible_concept
  {
    void constraints() {
      T a(b);            // require copy constructor
      T* ptr = &a;       // require address of operator
      const_constraints(a);
    }
    void const_constraints(const T& a) {
      T c(a);            // require const copy constructor
      const T* ptr = &a; // require const address of operator
    }
    T b;
  };

  template <class TT>
  struct EqualityComparable_concept
  {
    void constraints() {
      r = a == b;         // require equality operator
      r = a != b;         // require inequality operator
    }
    TT a, b;
    bool r;
  };

  template <class TT>
  struct LessThanComparable_concept
  {
    void constraints() {
      // require comparison operators
      r = a < b || a > b || a <= b || a >= b;
    }
    TT a, b;
    bool r;
  };

  template <class TT>
  struct TrivialIterator_concept
  {
    CLASS_REQUIRES(TT, Assignable);
    CLASS_REQUIRES(TT, DefaultConstructible);
    CLASS_REQUIRES(TT, EqualityComparable);

    void constraints() {
      typedef typename std::iterator_traits<TT>::value_type V;
      (void)*i;           // require dereference operator
    }
    TT i;
  };

  template <class TT>
  struct Mutable_TrivialIterator_concept
  {
    CLASS_REQUIRES(TT, TrivialIterator);
    void constraints() {
      *i = *j;            // require dereference and assignment
    }
    TT i, j;
  };

  template <class TT>
  struct InputIterator_concept
  {
    CLASS_REQUIRES(TT, TrivialIterator);
    void constraints() {
      // require iterator_traits typedef's
      typedef typename std::iterator_traits<TT>::difference_type D;
      typedef typename std::iterator_traits<TT>::reference R;
      typedef typename std::iterator_traits<TT>::pointer P;
      typedef typename std::iterator_traits<TT>::iterator_category C;
      REQUIRE2(typename std::iterator_traits<TT>::iterator_category,
               std::input_iterator_tag, Convertible);
      ++i;                // require preincrement operator
      i++;                // require postincrement operator
    }
    TT i;
  };

  template <class TT, class ValueT>
  struct OutputIterator_concept
  {
    CLASS_REQUIRES(TT, Assignable);
    void constraints() {
      ++i;                // require preincrement operator
      i++;                // require postincrement operator
      *i++ = t;           // require postincrement and assignment
    }
    TT i, j;
    ValueT t;
  };

  template <class TT>
  struct ForwardIterator_concept
  {
    CLASS_REQUIRES(TT, InputIterator);
    void constraints() {
      REQUIRE2(typename std::iterator_traits<TT>::iterator_category,
               std::forward_iterator_tag, Convertible);
    }
  };

  template <class TT>
  struct Mutable_ForwardIterator_concept
  {
    CLASS_REQUIRES(TT, ForwardIterator);
    void constraints() {
      *i++ = t;         // require postincrement and assignment
    }
    TT i;
    typename std::iterator_traits<TT>::value_type t;
  };

  template <class TT>
  struct BidirectionalIterator_concept
  {
    CLASS_REQUIRES(TT, ForwardIterator);
    void constraints() {
      REQUIRE2(typename std::iterator_traits<TT>::iterator_category,
               std::bidirectional_iterator_tag, Convertible);
      --i;                // require predecrement operator
      i--;                // require postdecrement operator
    }
    TT i;
  };

  template <class TT>
  struct Mutable_BidirectionalIterator_concept
  {
    CLASS_REQUIRES(TT, BidirectionalIterator);
    CLASS_REQUIRES(TT, Mutable_ForwardIterator);
    void constraints() {
      *i-- = *i;                  // require postdecrement and assignment
    }
    TT i;
  };


  template <class TT>
  struct RandomAccessIterator_concept
  {
    CLASS_REQUIRES(TT, BidirectionalIterator);
    CLASS_REQUIRES(TT, LessThanComparable);

    void constraints() {
      REQUIRE2(typename std::iterator_traits<TT>::iterator_category,
        std::random_access_iterator_tag, Convertible);

      typedef typename std::iterator_traits<TT>::reference R;

      i += n;             // require assignment addition operator
      i = i + n; i = n + i; // require addition with difference type
      i -= n;             // require assignment subtraction operator
      i = i - n;                  // require subtraction with difference type
      n = i - j;                  // require difference operator
      (void)i[n];                 // require element access operator
    }
    TT a, b;
    TT i, j;
    typename std::iterator_traits<TT>::difference_type n;
  };

  template <class TT>
  struct Mutable_RandomAccessIterator_concept
  {
    CLASS_REQUIRES(TT, RandomAccessIterator);
    CLASS_REQUIRES(TT, Mutable_BidirectionalIterator);
    void constraints() {
      i[n] = *i;                  // require element access and assignment
    }
    TT i;
    typename std::iterator_traits<TT>::difference_type n;
  };


  template <class _Func, class _Ret>
  struct Generator_concept
  {
    void constraints() {
      __r = __f();   // require operator() member function
    }
    _Func __f;
    _Ret __r;
  };


  template <class _Func>
  struct Generator_concept<_Func,void>
  {
    void constraints() {
      __f();              // require operator() member function
    }
    _Func __f;
  };


  template <class _Func, class _Ret, class _Arg>
  struct UnaryFunction_concept
  {
    void constraints() {
      __r = __f(__arg); // require operator()
    }
    _Func __f;
    _Arg __arg;
    _Ret __r;
  };

  template <class _Func, class _Arg>
  struct UnaryFunction_concept<_Func,void,_Arg> {
    void constraints() { 
      __f(__arg);                 // require operator()
    }
    _Func __f;
  };

  template <class _Func, class _Ret, class _First, class _Second>
  struct BinaryFunction_concept
  {
    void constraints() { 
      __r = __f(__first, __second); // require operator()
    }
    _Func __f;
    _First __first;
    _Second __second;
    _Ret __r;
  };

  template <class _Func, class _First, class _Second>
  struct BinaryFunction_concept<_Func,void,_First,_Second>
  {
    void constraints() {
      __f(__first, __second); // require operator()
    }
    _Func __f;
    _First __first;
    _Second __second;
  };

  template <class _Func, class _Arg>
  struct UnaryPredicate_concept
  {
    void constraints() {
      __r = __f(__arg); // require operator() returning bool
    }
    _Func __f;
    _Arg __arg;
    bool __r;
  };

  template <class _Func, class _First, class _Second>
  struct BinaryPredicate_concept
  {
    void constraints() {
      __r =  __f(__a, __b); // require operator() returning bool
    }
    _Func __f;
    _First __a;
    _Second __b;
    bool __r;
  };

  // use this when functor is used inside a container class like std::set
  template <class _Func, class _First, class _Second>
  struct Const_BinaryPredicate_concept {
    CLASS_REQUIRES3(_Func,_First,_Second, BinaryPredicate);
    void constraints() { 
      bool __r =  __f(__a,__b); // operator() must be a const member function
    }
    _Func __f;
    _First __a;
    _Second __b;
  };

#define __STL_DEFINE_BINARY_OPERATOR_CONSTRAINT(_OP,_NAME) \
  template <class _Ret, class _First, class _Second> \
  struct _NAME##_concept { \
    void constraints() { (void)_constraints(); } \
    _Ret _constraints() {  \
      return __a _OP __b; \
    } \
    _First __a; \
    _Second __b; \
  }

__STL_DEFINE_BINARY_OPERATOR_CONSTRAINT(==, _OP_EQUAL);
__STL_DEFINE_BINARY_OPERATOR_CONSTRAINT(!=, _OP_NOT_EQUAL);
__STL_DEFINE_BINARY_OPERATOR_CONSTRAINT(<, _OP_LESS_THAN);
__STL_DEFINE_BINARY_OPERATOR_CONSTRAINT(<=, _OP_LESS_EQUAL);
__STL_DEFINE_BINARY_OPERATOR_CONSTRAINT(>, _OP_GREATER_THAN);
__STL_DEFINE_BINARY_OPERATOR_CONSTRAINT(>=, _OP_GREATER_EQUAL);
__STL_DEFINE_BINARY_OPERATOR_CONSTRAINT(+, _OP_PLUS);
__STL_DEFINE_BINARY_OPERATOR_CONSTRAINT(*, _OP_TIMES);
__STL_DEFINE_BINARY_OPERATOR_CONSTRAINT(/, _OP_DIVIDE);
__STL_DEFINE_BINARY_OPERATOR_CONSTRAINT(-, _OP_SUBTRACT);
__STL_DEFINE_BINARY_OPERATOR_CONSTRAINT(%, _OP_MOD);

#endif /*GGCL_MSVCPP*/

#endif /* STL_CONCEPT_CHECKS3_H */

