//
//=======================================================================
// Copyright 1997, 1998, 1999, 2000 University of Notre Dame.
// Authors: Andrew Lumsdaine, Lie-Quan Lee, Jeremy G. Siek
//
// This file is part of the Generic Graph Component Library
//
// You should have received a copy of the License Agreement for the
// Generic Graph Component Library along with the software;  see the
// file LICENSE.  If not, contact Office of Research, University of Notre
// Dame, Notre Dame, IN  46556.
//
// Permission to modify the code and to distribute modified code is
// granted, provided the text of this NOTICE is retained, a notice that
// the code was modified is included with the above COPYRIGHT NOTICE and
// with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
// file is distributed with the modified code.
//
// LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
// By way of example, but not limitation, Licensor MAKES NO
// REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
// PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
// OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
// OR OTHER RIGHTS.
//=======================================================================
//
#ifndef GGCL_RECURSIVE_DFS_H
#define GGCL_RECURSIVE_DFS_H

#include <boost/graph_structure.hpp>
#include <boost/property_accessor.hpp>
#include <ggcl/ggcl_config.h>
#include <ggcl/properties.h>
#include <ggcl/dfs_visitor.h>
#include <ggcl/bfs.h>

namespace ggcl {

  // A recursive version of depth-first search
  template <class Graph, class Visitor, class Color>
  void
  _depth_first_search(Graph& G, Visitor v, Color c)
  {
    typedef typename boost::graph_traits<Graph>::vertex_descriptor Vertex;
    REQUIRE2(Color, Vertex, ReadWritePropertyAccessor);
    typedef typename boost::property_traits<Color>::value_type ColorValue;
    REQUIRE(ColorValue, ColorValue);
    dfs_visitor<Color, Visitor> visitor(c, v);
    initialize_graph(G, visitor);
    typedef typename boost::graph_traits<Graph>::vertex_iterator Iter;
    Iter ui, ui_end;
    for (boost::tie(ui,ui_end) = vertices(G); ui != ui_end; ++ui ) {
      if ( visitor.is_undiscovered(*ui) ) {
	visitor.start(*ui);
	depth_first_visit(G, *ui, visitor); //visitor is pass-by-reference!
      }
    }
  }

  // Variant (2)
  template <class Graph, class Visitor, class Color>
  void
  depth_first_search(Graph& G, Visitor v, Color c)
  {
    typedef typename boost::graph_traits<Graph>::vertex_descriptor Vertex;
    typedef typename boost::graph_traits<Graph>::edge_descriptor Edge;
    REQUIRE(Graph, VertexListGraph);
    REQUIRE4(Visitor, Vertex, Edge, Graph, UserVisitor);
    Vertex u = *vertices(G).first;
    _depth_first_search(G, v, adapt_property_accessor(c, boost::get(c,u), u));
  }

  // Variant (1)
  template <class Graph, class Visitor>
  inline void
  depth_first_search(Graph& G, Visitor v)
  {
    typedef typename boost::graph_traits<Graph>::vertex_descriptor Vertex;
    depth_first_search(G, v, get_vertex_property(G, color_tag()));
  }

  // depth_first_visit, the main component for recursive version of DFS.
  // This is the main component for the recursive version of DFS and
  // other DFS related algorithm patterns.
  template < class Graph, class Vertex, class Visitor >
  void
  depth_first_visit(Graph& G, Vertex u, Visitor& v)
  {
    REQUIRE(Graph, IncidenceGraph);
    typedef typename boost::graph_traits<Graph>::edge_descriptor Edge;
    REQUIRE4(Visitor, Vertex, Edge, Graph, Visitor);
    v.discover(u);
    typedef typename boost::graph_traits<Graph>::out_edge_iterator Iter;
    Iter i, iend;
    for (boost::tie(i,iend)=out_edges(u, G); i != iend; ++i)
      if ( v.process(*i, G) )
	depth_first_visit(G, target(*i, G), v );
    v.finish(u);
  }

}


#endif
