//
//=======================================================================
// Copyright 1997, 1998, 1999, 2000 University of Notre Dame.
// Authors: Andrew Lumsdaine, Lie-Quan Lee, Jeremy G. Siek
//
// This file is part of the Generic Graph Component Library
//
// You should have received a copy of the License Agreement for the
// Generic Graph Component Library along with the software;  see the
// file LICENSE.  If not, contact Office of Research, University of Notre
// Dame, Notre Dame, IN  46556.
//
// Permission to modify the code and to distribute modified code is
// granted, provided the text of this NOTICE is retained, a notice that
// the code was modified is included with the above COPYRIGHT NOTICE and
// with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
// file is distributed with the modified code.
//
// LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
// By way of example, but not limitation, Licensor MAKES NO
// REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
// PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
// OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
// OR OTHER RIGHTS.
//=======================================================================
//
#ifndef GGCL_DFS_VISITOR_H
#define GGCL_DFS_VISITOR_H

#include <boost/graph_structure.hpp>
#include <ggcl/utils.h>
#include <ggcl/properties.h>
#include <ggcl/foundation_visitor_aux.h>


namespace ggcl {

  template < class Color, class Super >
  class  dfs_visitor {
  public:
    dfs_visitor(Color _c, const Super& s) 
      : base(s), c(_c) {}


    template <class Vertex>
    void initialize(Vertex u) {
      c[u] = white(c[u]);
      visitor_initialize_evaluate( base, u );
    }

    template <class Vertex>
    inline void start(Vertex u) {
      visitor_start_evaluate( base, u );
    }

    template <class Vertex>
    void discover(Vertex u) { 
      c[u] = gray(c[u]);
      visitor_discover_evaluate( base, u );
    }

    template <class Edge, class Graph>
    bool process(Edge e, Graph& G) {
      typedef typename boost::graph_traits<Graph>::vertex_descriptor Vertex;
      Vertex dest = target(e, G);
      bool ret = is_undiscovered(dest);
      if (ret) {
	visitor_process_evaluate(base, e, G, tree_edge_tag());
      } else if ( c[dest] == gray(c[dest]) ) {
	visitor_process_evaluate(base, e, G, back_edge_tag());
      } else  /*only happened for directed graph*/
	visitor_process_evaluate(base, e, G, forward_or_cross_edge_tag());
 
      visitor_process_evaluate(base, e, G, every_edge_tag());
      return ret;
    }

    template <class Vertex>
    void finish(Vertex u) {
      c[u] = black(c[u]);
      visitor_finish_evaluate( base, u ); 
    }

    template <class Vertex>
    bool is_undiscovered(Vertex u) {
      return c[u] == white(c[u]);
    }
  protected:
    Super base;
    Color c;
  };

}

#endif
