//
//=======================================================================
// Copyright 1997, 1998, 1999, 2000 University of Notre Dame.
// Authors: Andrew Lumsdaine, Lie-Quan Lee, Jeremy G. Siek
//
// This file is part of the Generic Graph Component Library
//
// You should have received a copy of the License Agreement for the
// Generic Graph Component Library along with the software;  see the
// file LICENSE.  If not, contact Office of Research, University of Notre
// Dame, Notre Dame, IN  46556.
//
// Permission to modify the code and to distribute modified code is
// granted, provided the text of this NOTICE is retained, a notice that
// the code was modified is included with the above COPYRIGHT NOTICE and
// with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
// file is distributed with the modified code.
//
// LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
// By way of example, but not limitation, Licensor MAKES NO
// REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
// PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
// OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
// OR OTHER RIGHTS.
//=======================================================================
//
#ifndef GGCL_DIJKSTRA_H
#define GGCL_DIJKSTRA_H

#include <functional>
#include <ggcl/best_first_search.h>

namespace ggcl {

  //  Dijkstra's Algorithm solves the single-source shortest-paths
  //  problem on a weighted, directed graph G for the case in which
  //  all edge weights are nonnegative. 

  template <class Graph, class Vertex, class Visitor, 
            class Distance, class Weight, class Color, class ID,
            class D, class W>
  inline void
  _dijkstra_shortest_paths(Graph& G, Vertex  s,
                           Distance d, Weight w, Color color, ID id, 
                           Visitor visit, D, W)
  {
    std::less<D> compare;
    std::plus<W> combine;
    best_first_search(G, s, d, w, color, id, compare, combine, visit);
  }

  // Variant (4)
  template <class Graph, class Vertex, class Visitor, 
            class Distance, class Weight, class Color, class ID>
  inline void
  dijkstra_shortest_paths(Graph& G, Vertex s, 
                          Distance d, Weight w, Color c, ID id,
                          Visitor visit)
  {
    typename boost::graph_traits<Graph>::edge_descriptor 
      e = *out_edges(s, G).first;
    _dijkstra_shortest_paths(G, s, d, w, c, id, visit, 
                             boost::get(d,s), boost::get(w,e));
  }

  // Variant (3)
  template <class Graph, class Vertex, class Distance, class Visitor>
  inline void
  dijkstra_shortest_paths(Graph& G, Vertex s, Distance d, Visitor visit)
  {
    dijkstra_shortest_paths(G, s, d, 
                            get_edge_property(G, weight_tag()), 
                            get_vertex_property(G, color_tag()), 
                            get_vertex_property(G, id_tag()), 
                            visit);
  }

  // Variant (2)
  template <class Graph, class Vertex, class Distance>
  inline void
  dijkstra_shortest_paths(Graph& G, Vertex s, Distance d)
  {
    dijkstra_shortest_paths(G, s, d,
                            get_edge_property(G, weight_tag()), 
                            get_vertex_property(G, color_tag()), 
                            get_vertex_property(G, id_tag()), 
			    null_visitor());
  }

  // Variant (1)
  template <class Graph, class Vertex>
  inline void
  dijkstra_shortest_paths(Graph& G, Vertex s)
  {
    dijkstra_shortest_paths(G, s, get_vertex_property(G, distance_tag()),
                            null_visitor());
  }


} /*namespace*/

#endif /* GGCL_DIJKSTRA_H */
