//
//=======================================================================
// Copyright 1997, 1998, 1999, 2000 University of Notre Dame.
// Authors: Andrew Lumsdaine, Lie-Quan Lee, Jeremy G. Siek
//
// This file is part of the Generic Graph Component Library
//
// You should have received a copy of the License Agreement for the
// Generic Graph Component Library along with the software;  see the
// file LICENSE.  If not, contact Office of Research, University of Notre
// Dame, Notre Dame, IN  46556.
//
// Permission to modify the code and to distribute modified code is
// granted, provided the text of this NOTICE is retained, a notice that
// the code was modified is included with the above COPYRIGHT NOTICE and
// with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
// file is distributed with the modified code.
//
// LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
// By way of example, but not limitation, Licensor MAKES NO
// REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
// PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
// OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
// OR OTHER RIGHTS.
//=======================================================================
//
#ifndef GGCL_DISTANCE_VISITOR_H
#define GGCL_DISTANCE_VISITOR_H

#include <ggcl/properties.h>
#include <ggcl/utils.h>

namespace ggcl {

  // This visitor is used to calculate the distance for every verterx
  // from the reference vertex (source).
  // 
  template <class Distance>
  class distance_visitor 
    : public base_visitor<distance_visitor<Distance> > {
  public:

    typedef tree_edge_tag category;
    inline distance_visitor() { }
    inline distance_visitor(Distance dist) : d(dist) { }

    template <class Vertex>
    inline void initialize(Vertex u) {
      d[u] = numeric_limits_max(D());
    }

    template <class Vertex>
    inline void start(Vertex s) {
      d[s] = 0;
    }

    template <class Edge, class Graph>
    inline bool process(Edge e, Graph& g) {
      typedef typename boost::graph_traits<Graph>::vertex_descriptor Vertex;
      Vertex v = target(e, g);
      bool ret = ( d[v] == numeric_limits_max(D()) );
      if ( ret )  
	d[v] = d[source(e, g)] + 1;
      return ret;
    }
  protected:
    Distance d;
    typedef typename boost::property_traits<Distance>::value_type D;
  };

  template <class Distance>
  distance_visitor<typename detail::property_accessor_adaptor<Distance>::type>
  visit_distance(Distance d) {
	  typedef typename detail::property_accessor_adaptor<Distance>::type D;
    return distance_visitor<D>(d);
  }

#if !defined GGCL_MSVCPP
  template <class D>
  distance_visitor< random_access_iterator_property_accessor<D*,D,D&> >
  visit_distance(D* d) {
    typedef random_access_iterator_property_accessor<D*,D,D&> PA;
    return PA(d);
  }
#endif

  template <class D>
  distance_visitor< random_access_iterator_property_accessor<D*,D,D&> >
  visit_distance_ptr(D* d) {
    typedef random_access_iterator_property_accessor<D*,D,D&> PA;
    return PA(d);
  }

}

#endif /*GGCL_DISTANCE_VISITOR_H*/
