//
//=======================================================================
// Copyright 1997, 1998, 1999, 2000 University of Notre Dame.
// Authors: Andrew Lumsdaine, Lie-Quan Lee, Jeremy G. Siek
//
// This file is part of the Generic Graph Component Library
//
// You should have received a copy of the License Agreement for the
// Generic Graph Component Library along with the software;  see the
// file LICENSE.  If not, contact Office of Research, University of Notre
// Dame, Notre Dame, IN  46556.
//
// Permission to modify the code and to distribute modified code is
// granted, provided the text of this NOTICE is retained, a notice that
// the code was modified is included with the above COPYRIGHT NOTICE and
// with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
// file is distributed with the modified code.
//
// LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
// By way of example, but not limitation, Licensor MAKES NO
// REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
// PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
// OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
// OR OTHER RIGHTS.
//=======================================================================
//
#ifndef GGCL_FUNCTOR_H
#define GGCL_FUNCTOR_H

#include <ggcl/ggcl_config.h>

#include <functional>

namespace ggcl {

  template <class Decorator>
  class functor_greater {
  public:
    inline functor_greater(const Decorator& df = Decorator())
      : d(df) { }

    //: return  <TT> d[u] &gt d[v] </TT>
    template <class Vertex>
    inline bool 
    operator()(const Vertex& u, const Vertex& v) const {
      return d[u] > d[v];
    }
  protected:
    Decorator d;
  };

  template <class Decorator>
  class functor_less {
  public:
    inline functor_less(const Decorator& df = Decorator())
      : d(df) { }
    //: return  <TT> d[u] &lt d[v] </TT>
    template <class Vertex>
    inline bool 
    operator()(const Vertex& u, const Vertex& v) const {
      return d[u] < d[v];
    }
  protected:
    Decorator d;
  };

  template <class Decorator>
  class functor_equal {
  public:
    inline functor_equal(const Decorator& df = Decorator()) 
      : d(df) { }
    //: return  <TT> d[u] == d[v] </TT>
    template <class Vertex>
    inline bool 
    operator()(const Vertex& u, const Vertex& v) const {
      return d[u] == d[v];
    }
  protected:
    Decorator d;
  };


  struct first_less {
    template < class PairType >
    inline bool operator()(const PairType& a, 
			   const PairType& b) const {
      return a.first < b.first;
    }
  };

  template < class PairType >
  struct first_equal : std::binary_function<PairType, PairType, bool>  {
    
    typedef typename std::binary_function<PairType, PairType, bool>::first_argument_type
                     first_argument_type;
    typedef typename std::binary_function<PairType, PairType, bool>::second_argument_type
                     second_argument_type;
    typedef typename std::binary_function<PairType, PairType, bool>::result_type
                     result_type;

    inline bool operator()(const PairType& a, 
			   const PairType& b) const {
      return a.first == b.first;
    }
  };

} /*namespace*/

#endif /*GGCL_FUNCTOR_H*/
