//
//=======================================================================
// Copyright 1997, 1998, 1999, 2000 University of Notre Dame.
// Authors: Andrew Lumsdaine, Lie-Quan Lee, Jeremy G. Siek
//
// This file is part of the Generic Graph Component Library
//
// You should have received a copy of the License Agreement for the
// Generic Graph Component Library along with the software;  see the
// file LICENSE.  If not, contact Office of Research, University of Notre
// Dame, Notre Dame, IN  46556.
//
// Permission to modify the code and to distribute modified code is
// granted, provided the text of this NOTICE is retained, a notice that
// the code was modified is included with the above COPYRIGHT NOTICE and
// with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
// file is distributed with the modified code.
//
// LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
// By way of example, but not limitation, Licensor MAKES NO
// REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
// PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
// OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
// OR OTHER RIGHTS.
//=======================================================================
//

#ifndef GGCL_GRAPH_REP_GEN_H
#define GGCL_GRAPH_REP_GEN_H

#include <ggcl/ggcl_config.h>

#include <vector>
#include <list>
#include <set>
#include <map>

#if defined GGCL_HAVE_HASH
#include <hash_set>
#include <hash_map>
#else
#include <contrib/hash_set>
#include <contrib/hash_map>
#endif

#if defined GGCL_HAVE_SLIST
#include <slist>
#endif

namespace ggcl {

#if defined GGCL_HAVE_SLIST
  struct slistS {};  
#endif

  struct vecS  {};
  struct listS {};
  struct mapS  {};
  struct hash_mapS {};

  struct allow_parallel_edge_tag {};
  struct disallow_parallel_edge_tag {};
}

#if defined GGCL_PARTIAL_SPECIALIZATION
namespace ggcl {
  template <class Selector, class ElemType>
  struct graph_rep_gen {
    typedef std::vector<ElemType> OneD;
    typedef std::vector<OneD>     TwoD;
    typedef allow_parallel_edge_tag  edge_parallel_category;
  };

  template <class ElemType>
  struct graph_rep_gen<mapS, ElemType> {
    typedef std::set<ElemType>    OneD;
    typedef std::vector<OneD>     TwoD;
    typedef disallow_parallel_edge_tag  edge_parallel_category;
  };

  template <class ElemType>
  struct graph_rep_gen<hash_mapS, ElemType> {
    typedef std::hash_set<ElemType> OneD;
    typedef std::vector<OneD>       TwoD;
    typedef disallow_parallel_edge_tag  edge_parallel_category;
  };

}

#define GGCL_GRAPH_REP_GEN_SPEC(Selector, oned, EP_CAT )   \
  template <class ElemType>                                \
  struct graph_rep_gen < Selector , ElemType > {           \
    typedef oned <ElemType>       OneD;                    \
    typedef std::vector<OneD>     TwoD;                    \
    typedef EP_CAT                edge_parallel_category;  \
  }

#else

#define GGCL_GRAPH_REP_GEN_SPEC(Selector, oned, EP_CAT) \
                                                        \
  template <>                                           \
  struct graph_rep_gen < Selector >                     \
  {                                                     \
    template < class ElemType >                         \
    struct bind {                                       \
      typedef oned < ElemType >              OneD;      \
      typedef std::vector< oned <ElemType> > TwoD;      \
      typedef EP_CAT             edge_parallel_category;  \
    };                                                  \
  }

namespace ggcl {

  template <typename Selector>
  struct graph_rep_gen {
    template <class ElemType >
    struct bind {
      typedef std::vector<ElemType> OneD;
      typedef std::vector<OneD>     TwoD;
      typedef allow_parallel_edge_tag  edge_parallel_category;
    };
  };


  template <> 
  struct graph_rep_gen < mapS >
  {
    template < class ElemType >
    struct bind {
      typedef std::set<ElemType> OneD;
      typedef std::vector<OneD>                        TwoD;
      typedef disallow_parallel_edge_tag  edge_parallel_category;
    };
  };

  template <> 
  struct graph_rep_gen < hash_mapS >
  {
    template < class ElemType >
    struct bind {
      typedef std::hash_set<ElemType> OneD;
      typedef std::vector<OneD>                        TwoD;
      typedef disallow_parallel_edge_tag  edge_parallel_category;
    };
  };
}

#endif //#if defined GGCL_PARTIAL_SPECIALIZATION

namespace ggcl {
#if defined GGCL_HAVE_SLIST
  GGCL_GRAPH_REP_GEN_SPEC(slistS, std::slist, allow_parallel_edge_tag);
#endif
  GGCL_GRAPH_REP_GEN_SPEC(listS, std::list, allow_parallel_edge_tag);
  GGCL_GRAPH_REP_GEN_SPEC(vecS, std::vector, allow_parallel_edge_tag);
}

#endif /*GGCL_GRAPH_REP_GEN_H*/
