#ifndef GGCL_JOHNSON_H
#define GGCL_JOHNSON_H

#include <ggcl/bellman_ford.h>
#include <ggcl/dijkstra.h>

namespace ggcl {

  template <class VertexAndEdgeListGraph, class DistanceMatrix,
            class Distance, class Weight, class Color, class VertexID>
  bool
  johnson_all_pairs_shortest_paths(VertexAndEdgeListGraph& g, 
                                   DistanceMatrix& D,
                                   Distance d, Distance h,
                                   Weight w, Color c, VertexID id)
  {
    typedef boost::graph_traits<VertexAndEdgeListGraph> Traits;
    typename Traits::vertex_iterator v, v_end, u, u_end;
    typename Traits::edge_iterator e, e_end;
    typename Traits::vertex_descriptor s = *vertices(g).first;
    typedef typename boost::property_traits<Distance>::value_type DT;

    d[s] = DT(); // initialize distance of source vertex to zero
    if (bellman_ford_shortest_paths(g, num_vertices(g), w, d)) {
      for (boost::tie(v,v_end) = vertices(g); v != v_end; ++v)
        h[*v] = d[*v];
      for (boost::tie(e,e_end) = edges(g); e != e_end; ++e)
        w[*e] = w[*e] + h[source(*e,g)] - h[target(*e,g)];
      for (boost::tie(u,u_end) = vertices(g); u != u_end; ++u) {
        dijkstra_shortest_paths(g, *u, d, w, c, id, null_visitor());
        for (boost::tie(v,v_end) = vertices(g); v != v_end; ++v)
          D[*u][*v] = d[*v] + h[*v] - h[*u];
      }
      return true;
    } else
      return false;
  }

} // namespace ggcl

#endif // GGCL_JOHNSON_H


