#ifndef GGCL_PREDECESSOR_GRAPH_H
#define GGCL_PREDECESSOR_GRAPH_H

#include <utility>
#include <iterator>
#include <boost/graph_structure.hpp>
#include <boost/property_accessor.hpp>
#include <ggcl/graph_rep_gen.h> // for allow_parallel_edge_tag

namespace ggcl {

  // This is a model of VertexListGraph
  template <class PredContainer>
  class predecessor_graph
  {
    typedef predecessor_graph self;
  public:
    typedef typename PredContainer::value_type vertex_descriptor;
    typedef std::pair<vertex_descriptor,vertex_descriptor> edge_descriptor;
    
    inline predecessor_graph(const PredContainer& p) : _p(p) { }
    
    struct out_edge_iterator {
    public:
      typedef out_edge_iterator self;
      typedef edge_descriptor value_type;
      typedef edge_descriptor reference;
      typedef edge_descriptor* pointer;
      typedef std::input_iterator_tag iterator_category;
      typedef typename PredContainer::difference_type difference_type;
      inline out_edge_iterator() { }
      inline out_edge_iterator(vertex_descriptor src, 
			       vertex_descriptor target, 
			       bool d)
        : _src(src), _target(target), _done(d) { }
      inline self& operator++() { _done = !_done; return *this; }
      inline self operator++(int) { self t = *this; _done = !_done; return t; }
      inline bool operator==(const self& x) const { return _done == x._done; }
      inline bool operator!=(const self& x) const { return _done != x._done; }
      inline edge_descriptor operator*() { 
	return std::make_pair(_src,_target); 
      }
    protected:
      vertex_descriptor _src, _target;
      bool _done;
    };
    typedef boost::int_iterator<vertex_descriptor> vertex_iterator;
    typedef void adjacency_iterator;
    typedef void in_edge_iterator;
    typedef void edge_iterator;
    typedef directed_tag directed_category;
    typedef allow_parallel_edge_tag edge_parallel_category;
    //  protected:  
    const PredContainer& _p;
  };

  template <class PredContainer>
  inline predecessor_graph<PredContainer>
  make_pred_graph(PredContainer p) {
    return predecessor_graph<PredContainer>(p);
  }

  // source(), target() already defined in edge.h  

  template <class Vertex, class P>
  std::pair<typename predecessor_graph<P>::out_edge_iterator,
            typename predecessor_graph<P>::out_edge_iterator>
  out_edges(Vertex u, const predecessor_graph<P>& g)
  {
    typedef typename predecessor_graph<P>::out_edge_iterator out_iter;
    return std::make_pair(out_iter(u, g._p[u], false), 
                          out_iter(u, g._p[u], true) );
  }

  template <class P>
  std::pair<typename predecessor_graph<P>::vertex_iterator,
            typename predecessor_graph<P>::vertex_iterator>
  vertices(const predecessor_graph<P>& g)
  {
    typedef typename predecessor_graph<P>::vertex_iterator viter;
    return std::make_pair(viter(0), viter(g._p.size()));
  }

  template <class P>
  id_property_accessor
  get_vertex_property(predecessor_graph<P>& g, id_tag) {
    return id_property_accessor();
  }
  template <class P>
  id_property_accessor
  get_vertex_property(const predecessor_graph<P>& g, id_tag) {
    return id_property_accessor();
  }

} // namespace ggcl

#endif
