//
//=======================================================================
// Copyright 1997, 1998, 1999, 2000 University of Notre Dame.
// Authors: Andrew Lumsdaine, Lie-Quan Lee, Jeremy G. Siek
//
// This file is part of the Generic Graph Component Library
//
// You should have received a copy of the License Agreement for the
// Generic Graph Component Library along with the software;  see the
// file LICENSE.  If not, contact Office of Research, University of Notre
// Dame, Notre Dame, IN  46556.
//
// Permission to modify the code and to distribute modified code is
// granted, provided the text of this NOTICE is retained, a notice that
// the code was modified is included with the above COPYRIGHT NOTICE and
// with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
// file is distributed with the modified code.
//
// LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
// By way of example, but not limitation, Licensor MAKES NO
// REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
// PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
// OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
// OR OTHER RIGHTS.
//=======================================================================
//
#ifndef GGCL_PREDECESSOR_VISITOR_H
#define GGCL_PREDECESSOR_VISITOR_H

#include <ggcl/properties.h>
#include <ggcl/utils.h>
#include <ggcl/foundation_visitor_aux.h>

namespace ggcl {

  // This is a visitor to record the predecessor of vertex discovered
  // in the graph algorithms.

  template <class Predecessor>
  class predecessor_visitor
    : public base_visitor< predecessor_visitor<Predecessor> > {
  public:
    typedef tree_edge_tag category;
    inline predecessor_visitor() { }
    inline predecessor_visitor(Predecessor _p) : p(_p) { }

    template <class Vertex>
    inline void initialize(Vertex u) {
      p[u] = Vertex(); 
    }

    template <class Vertex>
    inline void start(Vertex s) {
      p[s] = s;
    }

    template <class Edge, class Graph>
    inline bool process(Edge e, Graph& g) {
      p[target(e, g)] = source(e, g);
      return true;
    }
  protected:
    Predecessor p;
  };

  template <class Predecessor>
  inline predecessor_visitor<
    typename detail::property_accessor_adaptor<Predecessor>::type >
  visit_predecessor(Predecessor p) {
    typedef typename detail::property_accessor_adaptor<Predecessor>::type PA;
    return predecessor_visitor<PA>(p);
  }

#if !defined GGCL_MSVCPP
  template <class P>
  inline predecessor_visitor< 
    random_access_iterator_property_accessor<P*,P,P&> >
  visit_predecessor(P* p) {
    typedef random_access_iterator_property_accessor<P*,P,P&> PA;
    return predecessor_visitor<PA>(p);
  }
#endif

  template <class P>
  inline predecessor_visitor< 
    random_access_iterator_property_accessor<P*,P,P&> >
  visit_predecessor_ptr(P* p) {
    typedef random_access_iterator_property_accessor<P*,P,P&> PA;
    return predecessor_visitor<PA>(p);
  }

}


#endif /* GGCL_PREDECESSOR_VISITOR_H */
