//
//=======================================================================
// Copyright 1997, 1998, 1999, 2000 University of Notre Dame.
// Authors: Andrew Lumsdaine, Lie-Quan Lee, Jeremy G. Siek
//
// This file is part of the Generic Graph Component Library
//
// You should have received a copy of the License Agreement for the
// Generic Graph Component Library along with the software;  see the
// file LICENSE.  If not, contact Office of Research, University of Notre
// Dame, Notre Dame, IN  46556.
//
// Permission to modify the code and to distribute modified code is
// granted, provided the text of this NOTICE is retained, a notice that
// the code was modified is included with the above COPYRIGHT NOTICE and
// with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
// file is distributed with the modified code.
//
// LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
// By way of example, but not limitation, Licensor MAKES NO
// REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
// PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
// OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
// OR OTHER RIGHTS.
//=======================================================================
//
#ifndef GGCL_MST_PRIM_H
#define GGCL_MST_PRIM_H

#include <functional>
#include <ggcl/best_first_search.h>

namespace ggcl {
  

  template <class U, class V> struct _project2nd {
    V operator()(U, V v) const { return v; }
  };

  // This is Prim's algorithm to calculate the Minimum Spanning Tree
  // for an undirected graph with weighted edges.

  template <class Graph, class Vertex, class Visitor, 
            class Distance, class Weight, class Color, class ID,
            class D, class W>
  inline void
  _prim_minimum_spanning_tree(Graph& G, Vertex s,
			      Distance d, Weight w, Color c, ID id,
			      Visitor visit, D, W)
  {
    std::less<D> compare;
    _project2nd<W,W> combine;
    best_first_search(G, s, d, w, c, id, compare, combine, visit);    
  }

  // Variant (4)
  template <class Graph, class Vertex, class Visitor, 
            class Distance, class Weight, class Color, class ID>
  inline void
  prim_minimum_spanning_tree(Graph& G, Vertex s,
			     Distance d, Weight w, Color c, ID id,
                             Visitor visit)
  {
    typename boost::graph_traits<Graph>::edge_descriptor
      e = *out_edges(s, G).first;
    _prim_minimum_spanning_tree(G, s, d, w, c, id, visit,
				boost::get(d,s), boost::get(w,e));
  }

  // Variant (3)
  template <class Graph, class Vertex, class Distance, class Visitor>
  inline void
  prim_minimum_spanning_tree(Graph& G, Vertex s, Distance d, Visitor visit)
  {
    prim_minimum_spanning_tree(G, s, d, 
			       get_edge_property(G, weight_tag()), 
			       get_vertex_property(G, color_tag()), 
			       get_vertex_property(G, id_tag()), 
			       visit);
  }

  // Variant (2)
  template <class Graph, class Vertex, class Distance>
  inline void
  prim_minimum_spanning_tree(Graph& G, Vertex s, Distance d)
  {
    prim_minimum_spanning_tree(G, s, d, null_visitor());
  }

  // Variant (1)
  template <class Graph, class Vertex>
  inline void
  prim_minimum_spanning_tree(Graph& G, Vertex s)
  {
    prim_minimum_spanning_tree(G, s, get_vertex_property(G, distance_tag()));
  }

} /*namespace*/

#endif /*GGCL_MST_PRIM_H*/
