//
//=======================================================================
// Copyright 1997, 1998, 1999, 2000 University of Notre Dame.
// Authors: Andrew Lumsdaine, Lie-Quan Lee, Jeremy G. Siek
//
// This file is part of the Generic Graph Component Library
//
// You should have received a copy of the License Agreement for the
// Generic Graph Component Library along with the software;  see the
// file LICENSE.  If not, contact Office of Research, University of Notre
// Dame, Notre Dame, IN  46556.
//
// Permission to modify the code and to distribute modified code is
// granted, provided the text of this NOTICE is retained, a notice that
// the code was modified is included with the above COPYRIGHT NOTICE and
// with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
// file is distributed with the modified code.
//
// LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
// By way of example, but not limitation, Licensor MAKES NO
// REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
// PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
// OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
// OR OTHER RIGHTS.
//=======================================================================
//

#ifndef GGCL_RELAX_HPP
#define GGCL_RELAX_HPP

#include <functional>
#include <boost/graph_structure.hpp>

namespace ggcl {

  // warning: when weight of e is size_t and weight of e is
  // close to max of size_t, du + wuv will overflow. To avoid
  // this, Be careful about the value of weight. If it is larger
  // than the half of current max of the type, change the type of
  // weight to have more bytes.

  template <class Edge, class Graph, class Weight, class Distance, 
            class BinaryOperator, class Vertex, class Compare>
  bool _relax(Edge e, Graph& g, Weight w, Distance d, BinaryOperator op, 
        Compare cmp, Vertex)
  {
    Vertex u = source(e,g), v = target(e,g);// incident would be nicer
    
    if ( cmp(op(d[u], w[e]), d[v]) ) {
      d[v] = op(d[u], w[e]);
      return true;
    } else
      return false;
  }

  template <class Edge, class Graph, class Weight, class Distance, 
            class BinaryOperator, class Compare>
  bool relax(Edge e, Graph& g, Weight w, Distance d, 
	     BinaryOperator op, Compare cmp)
  {
    return _relax(e, g, w, d, op, cmp, target(e,g));
  }

  template <class Edge, class Graph, class Weight, class Distance, class D>
  bool _relax(Edge e, Graph& g, Weight w, Distance d, D)
  {
    typedef std::plus<D> Op;
    typedef std::less<D> Cmp;
    return relax(e, g, w, d, Op(), Cmp());
  }
  template <class Edge, class Graph, class Weight, class Distance>
  bool relax(Edge e, Graph& g, Weight w, Distance d)
  {
    return _relax(e, g, w, d, d[target(e,g)]);
  }

} /* namespace ggcl */

#endif /* GGCL_RELAX_HPP */
