//
//=======================================================================
// Copyright 1997, 1998, 1999, 2000 University of Notre Dame.
// Authors: Andrew Lumsdaine, Lie-Quan Lee, Jeremy G. Siek
//
// This file is part of the Generic Graph Component Library
//
// You should have received a copy of the License Agreement for the
// Generic Graph Component Library along with the software;  see the
// file LICENSE.  If not, contact Office of Research, University of Notre
// Dame, Notre Dame, IN  46556.
//
// Permission to modify the code and to distribute modified code is
// granted, provided the text of this NOTICE is retained, a notice that
// the code was modified is included with the above COPYRIGHT NOTICE and
// with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
// file is distributed with the modified code.
//
// LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
// By way of example, but not limitation, Licensor MAKES NO
// REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
// PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
// OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
// OR OTHER RIGHTS.
//=======================================================================
//
#ifndef GGCL_TIMESTAMP_VISITOR_H
#define GGCL_TIMESTAMP_VISITOR_H

#include <ggcl/utils.h>
#include <ggcl/foundation_visitor_aux.h>

namespace ggcl {

  // This visitor is to record the discover time and finish time of
  // vertices during graph traversal. Notice that only one timer for
  // both time.
  //
  template <class DiscoverTime, class FinishTime>
  class timestamp_visitor 
    : public base_visitor<timestamp_visitor<DiscoverTime, FinishTime> > {
  public:
    typedef tree_edge_tag category;
    inline timestamp_visitor() : time(0) { }
    inline timestamp_visitor(DiscoverTime disc, FinishTime fin) 
      : d(disc), f(fin), time(0) { }

    template <class Vertex>
    inline void discover(Vertex u) {
      d[u] = ++time;
    }

    template <class Vertex>
    inline void finish(Vertex u) {
      f[u] = ++time;
    }

  protected:
    DiscoverTime d;
    FinishTime f;
    int time;
  };

  template <class DiscoverTime, class FinishTime>
  inline timestamp_visitor<
  typename detail::property_accessor_adaptor<DiscoverTime>::type, 
  typename detail::property_accessor_adaptor<FinishTime>::type>
  visit_timestamp(DiscoverTime d, FinishTime f) {
    typedef typename detail::property_accessor_adaptor<DiscoverTime>::type 
      DiscoverTimeD;
    typedef typename detail::property_accessor_adaptor<FinishTime>::type
      FinishTimeD;
    return timestamp_visitor<DiscoverTimeD, FinishTimeD>(d, f);
  }

#if !defined GGCL_MSVCPP
  template <class D, class F>
  inline timestamp_visitor< 
    random_access_iterator_property_accessor<D*,D,D&>,
    random_access_iterator_property_accessor<F*,F,F&> >
  visit_timestamp(D* d, F* f) {
    typedef random_access_iterator_property_accessor<D*,D,D&> DPA;
    typedef random_access_iterator_property_accessor<F*,F,F&> FPA;
    return timestamp_visitor<DPA, FPA>(d, f);
  }
#endif

  template <class D, class F>
  inline timestamp_visitor< 
    random_access_iterator_property_accessor<D*,D,D&>,
    random_access_iterator_property_accessor<F*,F,F&> >
  visit_timestamp_ptr(D* d, F* f) {
    typedef random_access_iterator_property_accessor<D*,D,D&> DPA;
    typedef random_access_iterator_property_accessor<F*,F,F&> FPA;
    return timestamp_visitor<DPA, FPA>(d, f);
  }

}
#endif /*GGCL_TIMESTAMP_VISITOR_H*/
