//
//=======================================================================
// Copyright 1997, 1998, 1999, 2000 University of Notre Dame.
// Authors: Andrew Lumsdaine, Lie-Quan Lee, Jeremy G. Siek
//
// This file is part of the Generic Graph Component Library
//
// You should have received a copy of the License Agreement for the
// Generic Graph Component Library along with the software;  see the
// file LICENSE.  If not, contact Office of Research, University of Notre
// Dame, Notre Dame, IN  46556.
//
// Permission to modify the code and to distribute modified code is
// granted, provided the text of this NOTICE is retained, a notice that
// the code was modified is included with the above COPYRIGHT NOTICE and
// with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
// file is distributed with the modified code.
//
// LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
// By way of example, but not limitation, Licensor MAKES NO
// REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
// PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
// OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
// OR OTHER RIGHTS.
//=======================================================================
//
#ifndef GGCL_VISITED_H
#define GGCL_VISITED_H

namespace ggcl {


  // Null operation function object
  //
  //This functor provides three overloaded versions of
  //<TT>operator()</TT> function with one argument, two arguments, and
  //three arguments. The function body is empty to provide the null
  //operation.
  //
  struct null_operation {
    template < class A, class B, class C >
    inline void operator() ( const A& a, const B& b, const C& c ) const
    {}
    template < class A, class B >
    inline void operator() ( const A& a, const B& b ) const
    {}
    template < class A>
    inline void operator() ( const A& a ) const
    {}
  };

  // Queue update function object
  //
  // This functor is used for dijkstra amd prim. It provides a
  // operation to check if it needs perform queue update. if so, do
  // it.
  //
  struct queue_update {
    template < class Visitor, class Qtype, class Vertex >
    inline void operator() ( Visitor& visitor, Qtype& Q, Vertex u ) const
    {
      if ( visitor.need_update_queue() ) {
	Q.update(u);
	visitor.reset_need_update();
      }
    }
  };
  

}

#endif /*GGCL_VISITED_H*/
